@@ -57,7 +57,8 @@ arches = {
     "alpha":      "noarch",
 
     # arm
-    "armv61": "armv5tejl",
+    "armv7l": "armv6l",
+    "armv6l": "armv5tejl",
     "armv5tejl": "armv5tel",
     "armv5tel": "noarch",
 
@@ -87,7 +88,25 @@ def legitMultiArchesInSameLib(arch=None):
     return results        
 
 
+def canCoinstall(arch1, arch2):
+    """Take two arches and return True if it is possible that they can be
+       installed together with the same nevr. Ex: arch1=i386 and arch2=i686 then
+       it will return False. arch1=i386 and arch2=x86_64 will return True.
+       It does not determine whether or not the arches make any sense. Just whether
+       they could possibly install w/o conflict"""
 
+    # if both are a multlibarch then we can't coinstall  (x86_64, ia32e)
+    # if both are not multilibarches then we can't coinstall (i386, i686)
+    
+    if 'noarch' in [arch1, arch2]: # noarch can never coinstall
+        return False
+
+    if isMultiLibArch(arch=arch1) == isMultiLibArch(arch=arch2):
+        return False
+    # this section keeps arch1=x86_64 arch2=ppc from returning True
+    if arch1 in getArchList(arch2) or arch2 in getArchList(arch1):
+        return True
+    return False
 
 # this computes the difference between myarch and targetarch
 def archDifference(myarch, targetarch):
@@ -125,13 +144,14 @@ def getBestArchFromList(archlist, myarch=None):
         then return the best arch from the list for the canonArch.
     """
     
+    if len(archlist) == 0:
+        return None
+
     if myarch is None:
         myarch = canonArch
     
     mybestarch = getBestArch(myarch)
     
-    if len(archlist) == 0:
-        return None
     bestarch = getBestArch(myarch)
     if bestarch != myarch:
         bestarchchoice = getBestArchFromList(archlist, bestarch)
@@ -143,7 +163,7 @@ def getBestArchFromList(archlist, myarch=None):
         val1 = archDifference(myarch, thisarch)
         val2 = archDifference(myarch, arch)
         if val1 == 0 and val2 == 0:
-           continue
+            continue
         if val1 < val2:
             if val1 == 0:
                 thisarch = arch                
@@ -177,6 +197,9 @@ def getArchList(thisarch=None):
     if archlist[0] == "sparc64v":
         archlist.insert(1,"sparc64")
     
+    # if we're a weirdo arch - add noarch on there.
+    if len(archlist) == 1 and archlist[0] == thisarch:
+        archlist.append('noarch')
     return archlist
     
         
