@@ -25,7 +25,7 @@ import logging
 import rpmUtils.transaction
 import rpmUtils.miscutils
 import rpmUtils.arch
-from rpmUtils.arch import archDifference, isMultiLibArch, getBestArch
+from rpmUtils.arch import archDifference, isMultiLibArch, getBestArch, canCoinstall
 import misc
 from misc import unique, version_tuple_to_string
 import rpm
@@ -45,14 +45,14 @@ try:
     assert max(2, 4) == 4
 except:
     # Python-2.4.x doesn't have min/max ... *sigh*
-        def min(x, *args): 
-            for y in args:
-                if x > y: x = y
-            return x
-        def max(x, *args):
-            for y in args:
-                if x < y: x = y
-            return x
+    def min(x, *args): 
+        for y in args:
+            if x > y: x = y
+        return x
+    def max(x, *args):
+        for y in args:
+            if x < y: x = y
+        return x
 flags = {"GT": rpm.RPMSENSE_GREATER,
          "GE": rpm.RPMSENSE_EQUAL | rpm.RPMSENSE_GREATER,
          "LT": rpm.RPMSENSE_LESS,
@@ -85,25 +85,37 @@ class Depsolve(object):
                 Errors.YumFutureDeprecationWarning, stacklevel=2)
         return self._getTs()
         
-    def _getTs(self):
+    def _getTs(self, remove_only=False):
         """setup all the transaction set storage items we'll need
            This can't happen in __init__ b/c we don't know our installroot
            yet"""
         
         if self._tsInfo != None and self._ts != None:
+            if not remove_only and self._tsInfo.pkgSack is None:
+                self._tsInfo.setDatabases(self.rpmdb, self.pkgSack)
             return
             
         if not self.conf.installroot:
             raise Errors.YumBaseError, _('Setting up TransactionSets before config class is up')
         
-        self._getTsInfo()
+        self._getTsInfo(remove_only)
         self.initActionTs()
     
-    def _getTsInfo(self):
+    def _getTsInfo(self, remove_only=False):
+        """ remove_only param. says if we are going to do _only_ remove(s) in
+            the transaction. If so we don't need to setup the remote repos. """
         if self._tsInfo is None:
             self._tsInfo = self._transactionDataFactory()
-            self._tsInfo.setDatabases(self.rpmdb, self.pkgSack)
+            if remove_only:
+                pkgSack = None
+            else:
+                pkgSack = self.pkgSack
+            self._tsInfo.setDatabases(self.rpmdb, pkgSack)
             self._tsInfo.installonlypkgs = self.conf.installonlypkgs # this kinda sucks
+            # this REALLY sucks, sadly (needed for group conditionals)
+            self._tsInfo.install_method = self.install
+            self._tsInfo.update_method = self.update
+            self._tsInfo.remove_method = self.remove
         return self._tsInfo
 
     def _setTsInfo(self, value):
@@ -251,6 +263,17 @@ class Depsolve(object):
                 self.verbose_logger.log(logginglevels.DEBUG_1,
                     _('Removing Package %s'), txmbr.po)
 
+    def _dscb_procReq(self, po, niceformatneed):
+        """ Call the callback for processing requires, call the nicest one
+            available. """
+        if not self.dsCallback:
+            return
+
+        if hasattr(self.dsCallback, 'procReqPo'):
+            self.dsCallback.procReqPo(po, niceformatneed)
+        else:
+            self.dsCallback.procReq(po.name, niceformatneed)
+
     def _processReq(self, po, requirement):
         """processes a Requires dep from the resolveDeps functions, returns a tuple
            of (CheckDeps, missingdep, conflicts, errors) the last item is an array
@@ -261,7 +284,7 @@ class Depsolve(object):
         needname, flags, needversion = requirement
         niceformatneed = rpmUtils.miscutils.formatRequire(needname, needversion, flags)
         self.verbose_logger.log(logginglevels.DEBUG_1, _('%s requires: %s'), po, niceformatneed)
-        if self.dsCallback: self.dsCallback.procReq(po.name, niceformatneed)
+        self._dscb_procReq(po, niceformatneed)
 
         try:    
             if po.repo.id != "installed":
@@ -285,6 +308,13 @@ class Depsolve(object):
             missingdep = 0
 
         return (CheckDeps, missingdep, errormsgs)
+
+    @staticmethod
+    def _prco_req_nfv2req(rn, rf, rv):
+        return (rn, flags[rf], version_tuple_to_string(rv))
+
+    def _prco_req2req(self, req):
+        return self._prco_req_nfv2req(req[0], req[1], req[2])
             
     def _requiringFromInstalled(self, requiringPo, requirement, errorlist):
         """processes the dependency resolution for a dep where the requiring 
@@ -367,7 +397,8 @@ class Depsolve(object):
         
         if needmode in ['i', 'u']:
             length = len(self.tsInfo)
-            self.update(name=name, epoch=epoch, version=ver, release=rel)
+            self.update(name=name, epoch=epoch, version=ver, release=rel,
+                        requiringPo=requiringPo)
             txmbrs = self.tsInfo.getMembersWithState(requiringPo.pkgtup, TS_REMOVE_STATES)
             if len(self.tsInfo) != length and txmbrs:
                 if txmbrs[0].output_state == TS_OBSOLETED:
@@ -375,6 +406,11 @@ class Depsolve(object):
                                             requiringPo, txmbrs[0].obsoleted_by[0])
                 else:
                     self.verbose_logger.log(logginglevels.DEBUG_2, _('TSINFO: Updating %s to resolve dep.'), requiringPo)
+                    # If the requirement is still there, try and solve it again
+                    # so we don't lose it
+                    for pkg in txmbrs[0].updated_by:
+                        if requirement in map(self._prco_req2req, pkg.returnPrco('requires')):
+                            return True, missingdep + self._requiringFromTransaction(pkg, requirement, errorlist)[1]
                 checkdeps = True
                 return checkdeps, missingdep
             self.verbose_logger.log(logginglevels.DEBUG_2, _('Cannot find an update path for dep for: %s'), niceformatneed)
@@ -396,7 +432,24 @@ class Depsolve(object):
 
         return checkdeps, missingdep
         
+    def _quickWhatProvides(self, name, flags, version):
+        if self._last_req is None:
+            return False
 
+        if flags == 0:
+            flags = None
+        if type(version) in (types.StringType, types.NoneType, types.UnicodeType):
+            (r_e, r_v, r_r) = rpmUtils.miscutils.stringToVersion(version)
+        elif type(version) in (types.TupleType, types.ListType): # would this ever be a ListType?
+            (r_e, r_v, r_r) = version
+        
+        # Quick lookup, lots of reqs for one pkg:
+        po = self._last_req
+        if po.checkPrco('provides', (name, flags, (r_e, r_v, r_r))):
+            self.verbose_logger.debug(_('Quick matched %s to require for %s'), po, name)
+            return True
+        return False
+        
     def _requiringFromTransaction(self, requiringPo, requirement, errorlist):
         """processes the dependency resolution for a dep where requiring 
            package is in the transaction set"""
@@ -427,8 +480,10 @@ class Depsolve(object):
         #                   - if they are the same
         #                       - be confused but continue
 
-        provSack = self.whatProvides(needname, needflags, needversion)
+        if self._quickWhatProvides(needname, needflags, needversion):
+            return checkdeps, missingdep
 
+        provSack = self.whatProvides(needname, needflags, needversion)
         # get rid of things that are already in the rpmdb - b/c it's pointless to use them here
 
         for pkg in provSack.returnPackages():
@@ -445,20 +500,21 @@ class Depsolve(object):
             tspkgs = []
             if not self.allowedMultipleInstalls(pkg):
                 # from ts
-                tspkgs = self.tsInfo.matchNaevr(name=pkg.name, arch=pkg.arch)
+                tspkgs = self.tsInfo.matchNaevr(name=pkg.name)
                 for tspkg in tspkgs:
-                    if tspkg.po.verGT(pkg):
-                        msg = _('Potential resolving package %s has newer instance in ts.') % pkg
-                        self.verbose_logger.log(logginglevels.DEBUG_2, msg)
-                        provSack.delPackage(pkg)
-                        continue
-                    elif tspkg.po.verLT(pkg):
-                        upgraded.setdefault(pkg.pkgtup, []).append(tspkg.pkgtup)
+                    if not canCoinstall(pkg.arch, tspkg.po.arch): # a comparable arch
+                        if tspkg.po.verGT(pkg):
+                            msg = _('Potential resolving package %s has newer instance in ts.') % pkg
+                            self.verbose_logger.log(logginglevels.DEBUG_2, msg)
+                            provSack.delPackage(pkg)
+                            continue
+                        elif tspkg.po.verLT(pkg):
+                            upgraded.setdefault(pkg.pkgtup, []).append(tspkg.pkgtup)
                 
                 # from rpmdb
-                dbpkgs = self.rpmdb.searchNevra(name=pkg.name, arch=pkg.arch)
+                dbpkgs = self.rpmdb.searchNevra(name=pkg.name)
                 for dbpkg in dbpkgs:
-                    if dbpkg.verGT(pkg):
+                    if dbpkg.verGT(pkg) and not canCoinstall(pkg.arch, dbpkg.arch):
                         msg = _('Potential resolving package %s has newer instance installed.') % pkg
                         self.verbose_logger.log(logginglevels.DEBUG_2, msg)
                         provSack.delPackage(pkg)
@@ -483,8 +539,8 @@ class Depsolve(object):
                     _('%s already in ts, skipping this one'), pkg)
                 # FIXME: Remove this line, if it is not needed ?
                 # checkdeps = 1
+                self._last_req = pkg
                 return checkdeps, missingdep
-        
 
         # find the best one 
 
@@ -496,6 +552,7 @@ class Depsolve(object):
             for txmbr in results:
                 if pkg == txmbr.po:
                     checkdeps = True
+                    self._last_req = pkg
                     return checkdeps, missingdep
 
         # find out which arch of the ones we can choose from is closest
@@ -531,11 +588,14 @@ class Depsolve(object):
             txmbr = self.tsInfo.addUpdate(best, inst[0])
             txmbr.setAsDep(po=requiringPo)
             txmbr.reason = "dep"
+            self._last_req = best
         else:
             self.verbose_logger.debug(_('TSINFO: Marking %s as install for %s'), best,
                 requiringPo)
+            # FIXME: Don't we want .install() here, so obsoletes get done?
             txmbr = self.tsInfo.addInstall(best)
             txmbr.setAsDep(po=requiringPo)
+            self._last_req = best
 
             # if we had other packages with this name.arch that we found
             # before, they're not going to be installed anymore, so we
@@ -547,6 +607,16 @@ class Depsolve(object):
         
         return checkdeps, missingdep
 
+    def _dscb_procConflict(self, po, niceformatneed):
+        """ Call the callback for processing requires, call the nicest one
+            available. """
+        if not self.dsCallback:
+            return
+
+        if hasattr(self.dsCallback, 'procConflictPo'):
+            self.dsCallback.procConflictPo(po, niceformatneed)
+        else:
+            self.dsCallback.procConflict(po.name, niceformatneed)
 
     def _processConflict(self, po, conflict, conflicting_po):
         """processes a Conflict dep from the resolveDeps() method"""
@@ -558,7 +628,7 @@ class Depsolve(object):
         (name, arch, epoch, ver, rel) = po.pkgtup
 
         niceformatneed = rpmUtils.miscutils.formatRequire(needname, needversion, flags)
-        if self.dsCallback: self.dsCallback.procConflict(name, niceformatneed)
+        self._dscb_procConflict(po, niceformatneed)
 
         length = len(self.tsInfo)
         if flags & rpm.RPMSENSE_LESS:
@@ -620,18 +690,19 @@ class Depsolve(object):
         p.print_stats(20)
         return rc
 
-    def resolveDeps(self):
+    def resolveDeps(self, full_check=True):
 
         if not len(self.tsInfo):
             return (0, [_('Success - empty transaction')])
 
         self.po_with_problems = set()
         self._working_po = None
+        self._last_req = None
         self.tsInfo.resetResolved(hard=False)
 
         CheckDeps = True
-        CheckRemoves = False
-        CheckInstalls = False
+        CheckRemoves = full_check
+        CheckInstalls = full_check
 
         missingdep = 0
         errors = []
@@ -671,6 +742,8 @@ class Depsolve(object):
                     (checkdep, errormsgs) = self._processConflict(*conflict)
                     CheckDeps |= checkdep
                     errors += errormsgs
+                    if checkdep:
+                        break # The next conflict might be the same pkg
 
                 if CheckDeps:
                     if self.dsCallback: self.dsCallback.restartLoop()
@@ -749,6 +822,36 @@ class Depsolve(object):
 
         return CheckDeps, CheckInstalls, CheckRemoves, any_missing
 
+    @staticmethod
+    def _sort_reqs(pkgtup1, pkgtup2):
+        """ Sort the requires for a package from most "narrow" to least,
+            this tries to ensure that if we have two reqs like
+            "libfoo = 1.2.3-4" and "foo-api" (which is also provided by
+            libxyz-foo) that we'll get just libfoo.
+            There are other similar cases this "handles"."""
+
+        mapper = {'EQ' : 1, 'LT' : 2, 'LE' : 3, 'GT' : 4, 'GE' : 5,
+                  None : 99}
+        ret = mapper.get(pkgtup1[1], 10) - mapper.get(pkgtup2[1], 10)
+        if ret:
+            return ret
+
+        # This is pretty magic, basically we want an explicit:
+        #
+        #  Requires: foo
+        #
+        # ...to happen before the implicit:
+        #
+        #  Requires: libfoo.so.0()
+        #
+        # ...because sometimes the libfoo.so.0() is provided by multiple
+        # packages. Do we need more magic for other implicit deps. here?
+        def _req_name2val(name):
+            if (name.startswith("lib") and
+                (name.endswith("()") or name.endswith("()(64bit)"))):
+                return 99 # Processes these last
+            return 0
+        return _req_name2val(pkgtup1[0]) - _req_name2val(pkgtup2[0])
 
     def _checkInstall(self, txmbr):
         txmbr_reqs = txmbr.po.returnPrco('requires')
@@ -762,7 +865,7 @@ class Depsolve(object):
         oldreqs = set(oldreqs)
 
         ret = []
-        for req in txmbr_reqs:
+        for req in sorted(txmbr_reqs, cmp=self._sort_reqs):
             if req[0].startswith('rpmlib('):
                 continue
             if req in txmbr_provs:
@@ -773,7 +876,7 @@ class Depsolve(object):
             self.verbose_logger.log(logginglevels.DEBUG_2, _("looking for %s as a requirement of %s"), req, txmbr)
             provs = self.tsInfo.getProvides(*req)
             if not provs:
-                ret.append( (txmbr.po, (req[0], flags[req[1]], version_tuple_to_string(req[2]))) )
+                ret.append( (txmbr.po, self._prco_req2req(req)) )
                 continue
 
             #Add relationship
@@ -805,10 +908,12 @@ class Depsolve(object):
                 continue
             if newpoprovs.has_key(prov):
                 continue
+            # FIXME: This is probably the best place to fix the postfix rename
+            # problem long term (post .21) ... see compare_providers.
             for pkg, hits in self.tsInfo.getRequires(*prov).iteritems():
                 for rn, rf, rv in hits:
                     if not self.tsInfo.getProvides(rn, rf, rv):
-                        ret.append( (pkg, (rn, flags[rf], version_tuple_to_string(rv))) )
+                        ret.append( (pkg, self._prco_req_nfv2req(rn, rf, rv)) )
         return ret
 
     def _checkFileRequires(self):
@@ -876,7 +981,8 @@ class Depsolve(object):
                 for conflicting_po in self.tsInfo.getNewProvides(r, f, v):
                     if conflicting_po.pkgtup[0] == po.pkgtup[0] and conflicting_po.pkgtup[2:] == po.pkgtup[2:]:
                         continue
-                    ret.append( (po, (r, flags[f], version_tuple_to_string(v)), conflicting_po) )
+                    ret.append( (po, self._prco_req_nfv2req(r, f, v),
+                                 conflicting_po) )
         for txmbr in self.tsInfo.getMembersWithState(output_states=TS_INSTALL_STATES):
             po = txmbr.po
             for conflict in txmbr.po.returnPrco('conflicts'):
@@ -884,7 +990,8 @@ class Depsolve(object):
                 for conflicting_po in self.tsInfo.getProvides(r, f, v):
                     if conflicting_po.pkgtup[0] == po.pkgtup[0] and conflicting_po.pkgtup[2:] == po.pkgtup[2:]:
                         continue
-                    ret.append( (po, (r, flags[f], version_tuple_to_string(v)), conflicting_po) )
+                    ret.append( (po, self._prco_req_nfv2req(r, f, v),
+                                 conflicting_po) )
         return ret
 
 
@@ -910,6 +1017,12 @@ class Depsolve(object):
         self.verbose_logger.log(logginglevels.DEBUG_4,
               _("Running compare_providers() for %s") %(str(pkgs)))
 
+        def _cmp_best_providers(x, y):
+            """ Compare first by score, and then compare the pkgs if the score
+                is the same. Note that this sorts in reverse. """
+            ret = cmp(y[1], x[1])
+            if ret: return ret
+            return cmp(y[0], x[0])
         
         def _common_prefix_len(x, y, minlen=2):
             num = min(len(x), len(y))
@@ -950,9 +1063,31 @@ class Depsolve(object):
             return x
             
         pkgresults = {}
+        ipkgresults = {}
 
         for pkg in pkgs:
             pkgresults[pkg] = 0
+            if self.rpmdb.contains(pkg.name):
+                #  We only want to count things as "installed" if they are
+                # older than what we are comparing, because this then an update
+                # so we give preference. If they are newer then obsoletes/etc.
+                # could play a part ... this probably needs a better fix.
+                rpmdbpkgs = self.rpmdb.returnPackages(patterns=[pkg.name])
+                newest = sorted(rpmdbpkgs)[-1]
+                if newest.verLT(pkg):
+                    ipkgresults[pkg] = 0
+
+        #  This is probably only for "renames". What happens is that pkgA-1 gets
+        # obsoleted by pkgB but pkgB requires pkgA-2, now _if_ the pkgA txmbr
+        # gets processed before pkgB then we'll process the "checkRemove" of
+        # pkgA ... so any deps. on pkgA-1 will look for a new provider, one of
+        # which is pkgA-2 in that case we want to choose that pkg over any
+        # others. This works for multiple cases too, but who'd do that right?:)
+        #  FIXME: A good long term. fix here is to just not get into this
+        # problem, but that's too much for .21. This is much safer.
+        if ipkgresults:
+            pkgresults = ipkgresults
+            pkgs = ipkgresults.keys()
             
         # go through each pkg and compare to others
         # if it is same skip it
@@ -975,19 +1110,31 @@ class Depsolve(object):
             for nextpo in pkgs:
                 if po == nextpo:
                     continue
+
+                #  If this package isn't the latest version of said package,
+                # treat it like it's obsoleted. The problem here is X-1
+                # accidentally provides FOO, so you release X-2 without the
+                # provide, but X-1 is still picked over a real provider.
+                na = (po.name,po.arch)
+                lpos = self.pkgSack.returnNewestByNameArch(naTup=na)
+                if not lpos or po != sorted(lpos)[-1]:
+                    pkgresults[po] -= 1024
+
                 obsoleted = False
-                for obs in nextpo.obsoletes:
-                    if po.inPrcoRange('provides', obs):
-                        obsoleted = True
+                poprovtup = (po.name, 'EQ', (po.epoch, po.ver, po.release))
+                if nextpo.inPrcoRange('obsoletes', poprovtup):
+                    obsoleted = True
+                    pkgresults[po] -= 1024
                                 
-                        self.verbose_logger.log(logginglevels.DEBUG_4,
-                            _("%s obsoletes %s") % (po, nextpo))
+                    self.verbose_logger.log(logginglevels.DEBUG_4,
+                        _("%s obsoletes %s") % (nextpo, po))
 
-                    if obsoleted:
-                        pkgresults[po] -= 1024
-                        break
-
-                for thisarch in (reqpo.arch, getBestArch()):
+                if reqpo:
+                    arches = (reqpo.arch, getBestArch())
+                else:
+                    arches = (getBestArch(),)
+                
+                for thisarch in arches:
                     res = _compare_arch_distance(po, nextpo, thisarch)
                     if not res:
                         continue
@@ -1001,17 +1148,17 @@ class Depsolve(object):
                 self.verbose_logger.log(logginglevels.DEBUG_4,
                     _('common sourcerpm %s and %s' % (po, reqpo)))
                 pkgresults[po] += 20
-            
-            cpl = _common_prefix_len(po.name, reqpo.name)
-            if cpl > 2:
-                self.verbose_logger.log(logginglevels.DEBUG_4,
-                    _('common prefix of %s between %s and %s' % (cpl, po, reqpo)))
-            
-                pkgresults[po] += cpl*2
+            if reqpo:
+                cpl = _common_prefix_len(po.name, reqpo.name)
+                if cpl > 2:
+                    self.verbose_logger.log(logginglevels.DEBUG_4,
+                        _('common prefix of %s between %s and %s' % (cpl, po, reqpo)))
+                
+                    pkgresults[po] += cpl*2
             
             pkgresults[po] += (len(po.name)*-1)
 
-        bestorder = sorted(pkgresults.items(), key=itemgetter(1), reverse=True)
+        bestorder = sorted(pkgresults.items(), cmp=_cmp_best_providers)
         self.verbose_logger.log(logginglevels.DEBUG_4,
                 _('Best Order: %s' % str(bestorder)))
 
