@@ -9,6 +9,7 @@ from cStringIO import StringIO
 import base64
 import struct
 import re
+import errno
 import pgpmsg
 import tempfile
 import glob
@@ -16,9 +17,36 @@ import pwd
 import fnmatch
 import bz2
 from stat import *
-import gpgme
+try:
+    import gpgme
+    import gpgme.editutil
+except ImportError:
+    gpgme = None
+try:
+    import hashlib
+    _available_checksums = ['md5', 'sha1', 'sha256', 'sha512']
+    _default_checksums = ['sha256']
+except ImportError:
+    # Python-2.4.z ... gah!
+    import sha
+    import md5
+    _available_checksums = ['md5', 'sha1']
+    _default_checksums = ['sha1']
+    class hashlib:
+
+        @staticmethod
+        def new(algo):
+            if algo == 'md5':
+                return md5.new()
+            if algo == 'sha1':
+                return sha.new()
+            raise ValueError, "Bad checksum type"
 
 from Errors import MiscError
+# These are API things, so we can't remove them even if they aren't used here.
+# pylint: disable-msg=W0611
+from i18n import to_utf8, to_unicode
+# pylint: enable-msg=W0611
 
 _share_data_store   = {}
 _share_data_store_u = {}
@@ -66,6 +94,31 @@ def re_primary_filename(filename):
             return True
     return False
 
+_re_compiled_pri_dnames_match = None
+def re_primary_dirname(dirname):
+    global _re_compiled_pri_dnames_match
+    if _re_compiled_pri_dnames_match is None:
+        one   = re.compile('.*bin\/.*')
+        two   = re.compile('^\/etc\/.*')
+        _re_compiled_pri_dnames_match = (one, two)
+    for rec in _re_compiled_pri_dnames_match:
+        if rec.match(dirname):
+            return True
+    return False
+
+_re_compiled_full_match = None
+def re_full_search_needed(s):
+    """ Tests if a string needs a full nevra match, instead of just name. """
+    global _re_compiled_full_match
+    if _re_compiled_full_match is None:
+        one   = re.compile('.*[-\.\*\?\[\]].*.$') # Any wildcard or - seperator
+        two   = re.compile('^[0-9]')        # Any epoch, for envra
+        _re_compiled_full_match = (one, two)
+    for rec in _re_compiled_full_match:
+        if rec.match(s):
+            return True
+    return False
+
 ###########
 # Title: Remove duplicates from a sequence
 # Submitter: Tim Peters 
@@ -95,18 +148,16 @@ def unique(s):
     if n == 0:
         return []
 
-    # Try using a dict first, as that's the fastest and will usually
+    # Try using a set first, as that's the fastest and will usually
     # work.  If it doesn't work, it will usually fail quickly, so it
     # usually doesn't cost much to *try* it.  It requires that all the
     # sequence elements be hashable, and support equality comparison.
-    u = {}
     try:
-        for x in s:
-            u[x] = 1
+        u = set(s)
     except TypeError:
-        del u  # move on to the next method
+        pass
     else:
-        return u.keys()
+        return list(u)
 
     # We can't hash all the elements.  Second fastest is to sort,
     # which brings the equal elements together; then duplicates are
@@ -138,9 +189,94 @@ def unique(s):
             u.append(x)
     return u
 
-def checksum(sumtype, file, CHUNK=2**16):
+class Checksums:
+    """ Generate checksum(s), on given pieces of data. Producing the
+        Length and the result(s) when complete. """
+
+    def __init__(self, checksums=None, ignore_missing=False):
+        if checksums is None:
+            checksums = _default_checksums
+        self._sumalgos = []
+        self._sumtypes = []
+        self._len = 0
+
+        done = set()
+        for sumtype in checksums:
+            if sumtype == 'sha':
+                sumtype = 'sha1'
+            if sumtype in done:
+                continue
+
+            if sumtype in _available_checksums:
+                sumalgo = hashlib.new(sumtype)
+            elif ignore_missing:
+                continue
+            else:
+                raise MiscError, 'Error Checksumming, bad checksum type %s' % sumtype
+            done.add(sumtype)
+            self._sumtypes.append(sumtype)
+            self._sumalgos.append(sumalgo)
+        if not done:
+            raise MiscError, 'Error Checksumming, no valid checksum type'
+
+    def __len__(self):
+        return self._len
+
+    def update(self, data):
+        self._len += len(data)
+        for sumalgo in self._sumalgos:
+            sumalgo.update(data)
+
+    def read(self, fo, size=2**16):
+        data = fo.read(size)
+        self.update(data)
+        return data
+
+    def hexdigests(self):
+        ret = {}
+        for sumtype, sumdata in zip(self._sumtypes, self._sumalgos):
+            ret[sumtype] = sumdata.hexdigest()
+        return ret
+
+    def hexdigest(self, checksum=None):
+        if checksum is None:
+            checksum = self._sumtypes[0]
+        if checksum == 'sha':
+            checksum = 'sha1'
+        return self.hexdigests()[checksum]
+
+    def digests(self):
+        ret = {}
+        for sumtype, sumdata in zip(self._sumtypes, self._sumalgos):
+            ret[sumtype] = sumdata.digest()
+        return ret
+
+    def digest(self, checksum=None):
+        if checksum is None:
+            checksum = self._sumtypes[0]
+        if checksum == 'sha':
+            checksum = 'sha1'
+        return self.digests()[checksum]
+
+
+class AutoFileChecksums:
+    """ Generate checksum(s), on given file/fileobject. Pretending to be a file
+        object (overrrides read). """
+
+    def __init__(self, fo, checksums, ignore_missing=False):
+        self._fo       = fo
+        self.checksums = Checksums(checksums, ignore_missing)
+
+    def __getattr__(self, attr):
+        return getattr(self._fo, attr)
+
+    def read(self, size=-1):
+        return self.checksums.read(self._fo, size)
+
+
+def checksum(sumtype, file, CHUNK=2**16, datasize=None):
     """takes filename, hand back Checksum of it
-       sumtype = md5 or sha
+       sumtype = md5 or sha/sha1/sha256/sha512 (note sha == sha1)
        filename = /path/to/file
        CHUNK=65536 by default"""
      
@@ -150,25 +286,21 @@ def checksum(sumtype, file, CHUNK=2**16):
             fo = file # assume it's a file-like-object
         else:           
             fo = open(file, 'r', CHUNK)
-            
-        if sumtype == 'md5':
-            import md5
-            sumalgo = md5.new()
-        elif sumtype == 'sha':
-            import sha
-            sumalgo = sha.new()
-        else:
-            raise MiscError, 'Error Checksumming file, bad checksum type %s' % sumtype
-        chunk = fo.read
-        while chunk: 
-            chunk = fo.read(CHUNK)
-            sumalgo.update(chunk)
+
+        data = Checksums([sumtype])
+        while data.read(fo, CHUNK):
+            pass
 
         if type(file) is types.StringType:
             fo.close()
             del fo
             
-        return sumalgo.hexdigest()
+        # This screws up the length, but that shouldn't matter. We only care
+        # if this checksum == what we expect.
+        if datasize is not None and datasize > len(data):
+            return '!%u!%s' % (datasize, data.hexdigest(sumtype))
+
+        return data.hexdigest(sumtype)
     except (IOError, OSError), e:
         raise MiscError, 'Error opening file for checksum: %s' % file
 
@@ -187,8 +319,8 @@ def getFileList(path, ext, filelist):
             filelist = getFileList(path + '/' + d, ext, filelist)
         else:
             if d[-extlen:].lower() == '%s' % (ext):
-               newpath = os.path.normpath(path + '/' + d)
-               filelist.append(newpath)
+                newpath = os.path.normpath(path + '/' + d)
+                filelist.append(newpath)
                     
     return filelist
 
@@ -197,7 +329,14 @@ class GenericHolder:
        It exists purely to be able to do object.somestuff, object.someotherstuff
        or object[key] and pass object to another function that will 
        understand it"""
+
+    def __init__(self, iter=None):
+        self.__iter = iter
        
+    def __iter__(self):
+        if self.__iter is not None:
+            return iter(self[self.__iter])
+
     def __getitem__(self, item):
         if hasattr(self, item):
             return getattr(self, item)
@@ -321,26 +460,37 @@ def keyInstalled(ts, keyid, timestamp):
 
     return -1
 
-def import_key_to_pubring(rawkey, repo_cachedir):
-    gpgdir = '%s/gpgdir' % repo_cachedir
+def import_key_to_pubring(rawkey, keyid, cachedir=None, gpgdir=None):
+    # FIXME - cachedir can be removed from this method when we break api
+    if gpgme is None:
+        return False
+    
+    if not gpgdir:
+        gpgdir = '%s/gpgdir' % cachedir
+    
     if not os.path.exists(gpgdir):
         os.makedirs(gpgdir)
     
     key_fo = StringIO(rawkey) 
-    ctx = gpgme.Context()
     os.environ['GNUPGHOME'] = gpgdir
+    # import the key
+    ctx = gpgme.Context()
     fp = open(os.path.join(gpgdir, 'gpg.conf'), 'wb')
     fp.write('')
     fp.close()
     ctx.import_(key_fo)
     key_fo.close()
+    # ultimately trust the key or pygpgme is definitionally stupid
+    k = ctx.get_key(keyid)
+    gpgme.editutil.edit_trust(ctx, k, gpgme.VALIDITY_ULTIMATE)
+    return True
     
 def return_keyids_from_pubring(gpgdir):
-    ctx = gpgme.Context()
-    if not os.path.exists(gpgdir):
+    if gpgme is None or not os.path.exists(gpgdir):
         return []
-        
+
     os.environ['GNUPGHOME'] = gpgdir
+    ctx = gpgme.Context()
     keyids = []
     for k in ctx.keylist():
         for subkey in k.subkeys:
@@ -348,8 +498,38 @@ def return_keyids_from_pubring(gpgdir):
                 keyids.append(subkey.keyid)
 
     return keyids
-        
-def getCacheDir(tmpdir='/var/tmp'):
+
+def valid_detached_sig(sig_file, signed_file, gpghome=None):
+    """takes signature , file that was signed and an optional gpghomedir"""
+
+    if gpgme is None:
+        return False
+
+    if gpghome and os.path.exists(gpghome):
+        os.environ['GNUPGHOME'] = gpghome
+
+    sig = open(sig_file, 'r')
+    signed_text = open(signed_file, 'r')
+    plaintext = None
+    ctx = gpgme.Context()
+
+    try:
+        sigs = ctx.verify(sig, signed_text, plaintext)
+    except gpgme.GpgmeError, e:
+        return False
+    else:
+        # is there ever a case where we care about a sig beyond the first one?
+        thissig = sigs[0]
+        if not thissig:
+            return False
+
+        if thissig.validity in (gpgme.VALIDITY_FULL, gpgme.VALIDITY_MARGINAL,
+                                gpgme.VALIDITY_ULTIMATE):
+            return True
+
+    return False
+
+def getCacheDir(tmpdir='/var/tmp', reuse=True):
     """return a path to a valid and safe cachedir - only used when not running
        as root or when --tempcache is set"""
     
@@ -360,15 +540,17 @@ def getCacheDir(tmpdir='/var/tmp'):
     except KeyError:
         return None # if it returns None then, well, it's bollocksed
 
-    # check for /var/tmp/yum-username-* - 
-    prefix = 'yum-%s-' % username    
-    dirpath = '%s/%s*' % (tmpdir, prefix)
-    cachedirs = glob.glob(dirpath)
-    
-    for thisdir in cachedirs:
-        stats = os.lstat(thisdir)
-        if S_ISDIR(stats[0]) and S_IMODE(stats[0]) == 448 and stats[4] == uid:
-            return thisdir
+    prefix = 'yum-'
+
+    if reuse:
+        # check for /var/tmp/yum-username-* - 
+        prefix = 'yum-%s-' % username    
+        dirpath = '%s/%s*' % (tmpdir, prefix)
+        cachedirs = sorted(glob.glob(dirpath))
+        for thisdir in cachedirs:
+            stats = os.lstat(thisdir)
+            if S_ISDIR(stats[0]) and S_IMODE(stats[0]) == 448 and stats[4] == uid:
+                return thisdir
 
     # make the dir (tempfile.mkdtemp())
     cachedir = tempfile.mkdtemp(prefix=prefix, dir=tmpdir)
@@ -456,18 +638,10 @@ def get_running_kernel_version_release(ts):
     """This takes the output of uname and figures out the (version, release)
     tuple for the running kernel."""
     ver = os.uname()[2]
-    reduced = ver
-    # FIXME this should probably get passed this list from somewhere in config
-    # possibly from the kernelpkgnames option
-    for s in ("bigmem", "enterprise", "smp", "hugemem", "PAE", "rt",
-              "guest", "hypervisor", "xen0", "xenU", "xen", "debug",
-              "PAE-debug"):
-        if ver.endswith(s):
-            reduced = ver.replace(s, "")
-            
-    # we've got nothing so far, so... we glob for the file that MIGHT have
-    # this kernels and then look up the file in our rpmdb
-    fns = glob.glob('/boot/vmlinuz*%s*' % ver)
+
+    # we glob for the file that MIGHT have this kernel
+    # and then look up the file in our rpmdb.
+    fns = sorted(glob.glob('/boot/vmlinuz*%s*' % ver))
     for fn in fns:
         mi = ts.dbMatch('basenames', fn)
         for h in mi:
@@ -535,38 +709,112 @@ def find_ts_remaining(timestamp, yumlibpath='/var/lib/yum'):
         to_complete_items.append((action, pkgspec))
     
     return to_complete_items
+
+def seq_max_split(seq, max_entries):
+    """ Given a seq, split into a list of lists of length max_entries each. """
+    ret = []
+    num = len(seq)
+    seq = list(seq) # Trying to use a set/etc. here is bad
+    beg = 0
+    while num > max_entries:
+        end = beg + max_entries
+        ret.append(seq[beg:end])
+        beg += max_entries
+        num -= max_entries
+    ret.append(seq[beg:])
+    return ret
+
+def _ugly_utf8_string_hack(item):
+    """hands back a unicoded string"""
+    # this is backward compat for handling non-utf8 filenames 
+    # and content inside packages. :(
+    # content that xml can cope with but isn't really kosher
+
+    # if we're anything obvious - do them first
+    if item is None:
+        return ''
+    elif isinstance(item, unicode):    
+        return item
     
-def to_unicode(obj, encoding='utf-8', errors='replace'):
-    ''' convert a 'str' to 'unicode' '''
-    if isinstance(obj, basestring):
-        if not isinstance(obj, unicode):
-            obj = unicode(obj, encoding, errors)
-    return obj
-
-def to_utf8(obj, errors='replace'):
-    '''convert 'unicode' to an encoded utf-8 byte string '''
-    if isinstance(obj, unicode):
-        obj = obj.encode('utf-8', errors)
-    return obj
-
-# Don't use this, to_unicode should just work now
-def to_unicode_maybe(obj, encoding='utf-8', errors='replace'):
-    ''' Don't ask don't tell, only use when you must '''
+    # this handles any bogon formats we see
+    du = False
     try:
-        return to_unicode(obj, encoding, errors)
-    except UnicodeEncodeError:
-        return obj
+        x = unicode(item, 'ascii')
+        du = True
+    except UnicodeError:
+        encodings = ['utf-8', 'iso-8859-1', 'iso-8859-15', 'iso-8859-2']
+        for enc in encodings:
+            try:
+                x = unicode(item, enc)
+            except UnicodeError:
+                pass
+                
+            else:
+                if x.encode(enc) == item:
+                    if enc != 'utf-8':
+                        print '\n%s encoding on %s\n' % (enc, item)
+                    return x.encode('utf-8')
+    
+    
+    # Kill bytes (or libxml will die) not in the small byte portion of:
+    #  http://www.w3.org/TR/REC-xml/#NT-Char
+    # we allow high bytes, if it passed the utf8 check above. Eg.
+    # good chars = #x9 | #xA | #xD | [#x20-...]
+    newitem = ''
+    bad_small_bytes = range(0, 8) + [11, 12] + range(14, 32)
+    for char in item:
+        if ord(char) in bad_small_bytes:
+            pass # Just ignore these bytes...
+        elif not du and ord(char) > 127:
+            newitem = newitem + '?' # byte by byte equiv of escape
+        else:
+            newitem = newitem + char
+    return newitem
+
+def to_xml(item, attrib=False):
+    import xml.sax.saxutils
+    item = _ugly_utf8_string_hack(item)
+    item = to_utf8(item)
+    item = item.rstrip()
+    if attrib:
+        item = xml.sax.saxutils.escape(item, entities={'"':"&quot;"})
+    else:
+        item = xml.sax.saxutils.escape(item)
+    return item
+
+def unlink_f(filename):
+    """ Call os.unlink, but don't die if the file isn't there. This is the main
+        difference between "rm -f" and plain "rm". """
+    try:
+        os.unlink(filename)
+    except OSError, e:
+        if e.errno != errno.ENOENT:
+            raise
+
+# ---------- i18n ----------
+import locale
+import sys
+def setup_locale(override_codecs=True, override_time=False):
+    # This test needs to be before locale.getpreferredencoding() as that
+    # does setlocale(LC_CTYPE, "")
+    try:
+        locale.setlocale(locale.LC_ALL, '')
+        # set time to C so that we output sane things in the logs (#433091)
+        if override_time:
+            locale.setlocale(locale.LC_TIME, 'C')
+    except locale.Error, e:
+        # default to C locale if we get a failure.
+        print >> sys.stderr, 'Failed to set locale, defaulting to C'
+        os.environ['LC_ALL'] = 'C'
+        locale.setlocale(locale.LC_ALL, 'C')
+        
+    if override_codecs:
+        import codecs
+        sys.stdout = codecs.getwriter(locale.getpreferredencoding())(sys.stdout)
+        sys.stdout.errors = 'replace'
 
-def to_str(obj):
-    """ Convert something to a string, if it isn't one. """
-    # NOTE: unicode counts as a string just fine. We just want objects to call
-    # their __str__ methods.
-    if not isinstance(obj, basestring):
-        obj = str(obj)
-    return obj
 
 def get_my_lang_code():
-    import locale
     mylang = locale.getlocale()
     if mylang == (None, None): # odd :)
         mylang = 'C'
