@@ -30,8 +30,11 @@ import warnings
 from rpmUtils import RpmUtilsError
 import rpmUtils.arch
 import rpmUtils.miscutils
+from rpmUtils.miscutils import flagToString, stringToVersion
 import Errors
 import errno
+import struct
+from constants import *
 
 import urlparse
 urlparse.uses_fragment.append("media")
@@ -42,11 +45,21 @@ import grp
 
 def comparePoEVR(po1, po2):
     """
-    Compare two PackageEVR objects.
+    Compare two Package or PackageEVR objects.
     """
     (e1, v1, r1) = (po1.epoch, po1.version, po1.release)
     (e2, v2, r2) = (po2.epoch, po2.version, po2.release)
     return rpmUtils.miscutils.compareEVR((e1, v1, r1), (e2, v2, r2))
+def comparePoEVREQ(po1, po2):
+    """
+    Compare two Package or PackageEVR objects for equality.
+    """
+    (e1, v1, r1) = (po1.epoch, po1.version, po1.release)
+    (e2, v2, r2) = (po2.epoch, po2.version, po2.release)
+    if r1 != r2: return False
+    if v1 != v2: return False
+    if e1 != e2: return False
+    return True
 
 def buildPkgRefDict(pkgs, casematch=True):
     """take a list of pkg objects and return a dict the contains all the possible
@@ -139,10 +152,46 @@ def parsePackages(pkgs, usercommands, casematch=0,
         raise ValueError, "Bad value for unique: %s" % unique
     return exactmatch, matched, unmatched
 
+class FakeSack:
+    """ Fake PackageSack to use with FakeRepository"""
+    def __init__(self):
+        pass # This is fake, so do nothing
+    
+    def delPackage(self, obj):
+        """delete a pkgobject, do nothing, but make localpackages work with --skip-broken"""
+        pass # This is fake, so do nothing
+            
 class FakeRepository:
     """Fake repository class for use in rpmsack package objects"""
+
+    def _set_cleanup_repoid(self, repoid):
+        """ Set the repoid, but because it can be random ... clean it up. """
+
+        #  We don't want repoids to contain random bytes that can be
+        # in the FS directories. It's also nice if they aren't "huge". So
+        # just chop to the rpm name.
+        pathbased = False
+        if '/' in repoid:
+            repoid = os.path.basename(repoid)
+            pathbased = True
+
+        if repoid.endswith(".rpm"):
+            repoid = repoid[:-4]
+            pathbased = True
+
+        bytes = [] # Just in case someone uses mv to be evil:
+        if pathbased:
+            bytes.append('/')
+
+        for byte in repoid:
+            if ord(byte) >= 128:
+                byte = '?'
+            bytes.append(byte)
+        self.id = "".join(bytes)
+
     def __init__(self, repoid):
-        self.id = repoid
+        self._set_cleanup_repoid(repoid)
+        self.sack = FakeSack()
 
     def __cmp__(self, other):
         if self.id > other.id:
@@ -205,10 +254,29 @@ class PackageObject(object):
         if ret == 0 and hasattr(self, 'repoid') and hasattr(other, 'repoid'):
             ret = cmp(self.repoid, other.repoid)
         return ret
+    def __eq__(self, other):
+        """ Compare packages for yes/no equality, includes everything in the
+            UI package comparison. """
+        if not other:
+            return False
+        if self.pkgtup != other.pkgtup:
+            return False
+        if self.repoid != other.repoid:
+            return False
+        return True
+    def __ne__(self, other):
+        if not (self == other):
+            return True
+        return False
 
     def verEQ(self, other):
-        """ Uses verCMP, tests if the _rpm-versions_ are the same. """
-        return self.verCMP(other) == 0
+        """ Compare package to another one, only rpm-version equality. """
+        if not other:
+            return False
+        ret = cmp(self.name, other.name)
+        if ret != 0:
+            return False
+        return comparePoEVREQ(self, other)
     def verLT(self, other):
         """ Uses verCMP, tests if the other _rpm-version_ is <  ours. """
         return self.verCMP(other) <  0
@@ -257,22 +325,21 @@ class RpmBase(object):
         self.licenses = []
         self._hash = None
 
-    #  Do we still need __eq__ and __ne__ given that
-    # PackageObject has a working __cmp__?
+    # FIXME: This is identical to PackageObject.__eq__ and __ne__, should be
+    #        remove (is .repoid fine here? ... we need it, maybe .repo.id).
     def __eq__(self, other):
         if not other: # check if other not is a package object. 
             return False
-        if comparePoEVR(self, other) == 0 and self.arch == other.arch and self.name == other.name:
+        if self.pkgtup != other.pkgtup:
+            return False
+        if self.repoid != other.repoid:
+            return False
+        return True
+    def __ne__(self, other):
+        if not (self == other):
             return True
         return False
 
-    def __ne__(self, other):
-        if not other:
-            return True
-        if comparePoEVR(self, other) != 0 or self.arch != other.arch or self.name != other.name:
-            return True
-        return False
-       
     def returnEVR(self):
         return PackageEVR(self.epoch, self.version, self.release)
     
@@ -430,7 +497,9 @@ class PackageEVR:
     def __init__(self,e,v,r):
         self.epoch = e
         self.ver = v
+        self.version = v
         self.rel = r
+        self.release = r
         
     def compare(self,other):
         return rpmUtils.miscutils.compareEVR((self.epoch, self.ver, self.rel), (other.epoch, other.ver, other.rel))
@@ -457,12 +526,10 @@ class PackageEVR:
         return False
 
     def __eq__(self, other):
-        if self.compare(other) == 0:
-            return True
-        return False
+        return comparePoEVREQ(self, other)
 
     def __ne__(self, other):
-        if self.compare(other) != 0:
+        if not (self == other):
             return True
         return False
     
@@ -480,6 +547,7 @@ class YumAvailablePackage(PackageObject, RpmBase):
         self.repo = repo
         self.state = None
         self._loadedfiles = False
+        self._verify_local_pkg_cache = None
 
         if pkgdict != None:
             self.importFromDict(pkgdict)
@@ -516,10 +584,13 @@ class YumAvailablePackage(PackageObject, RpmBase):
         you should use self.repo.getPackage."""
         base = self.basepath
         if base:
+            # urljoin sucks in the reverse way that os.path.join sucks :)
+            if base[-1] != '/':
+                base = base + '/'
             return urlparse.urljoin(base, self.remote_path)
         return urlparse.urljoin(self.repo.urls[0], self.remote_path)
-    
-    size = property(_size)
+
+    size = property(fget=lambda self: self._size())
     remote_path = property(_remote_path)
     remote_url = property(_remote_url)
 
@@ -565,6 +636,12 @@ class YumAvailablePackage(PackageObject, RpmBase):
         return self._committime_ret
 
     committime = property(_committime)
+    
+    # FIXME test this to see if it causes hell elsewhere
+    def _checksum(self):
+        "Returns the 'default' checksum"
+        return self.checksums[0][1]
+    checksum = property(_checksum)
 
     def getDiscNum(self):
         if self.basepath is None:
@@ -619,17 +696,35 @@ class YumAvailablePackage(PackageObject, RpmBase):
     def verifyLocalPkg(self):
         """check the package checksum vs the localPkg
            return True if pkg is good, False if not"""
-           
+
+        #  This is called a few times now, so we want some way to not have to
+        # read+checksum "large" datasets multiple times per. transaction.
+        try:
+            nst = os.stat(self.localPkg())
+        except OSError, e:
+            return False
+        if (hasattr(self, '_verify_local_pkg_cache') and
+            self._verify_local_pkg_cache):
+            ost = self._verify_local_pkg_cache
+            if (ost.st_ino   == nst.st_ino   and
+                ost.st_dev   == nst.st_dev   and
+                ost.st_mtime == nst.st_mtime and
+                ost.st_size  == nst.st_size):
+                return True
+
         (csum_type, csum) = self.returnIdSum()
         
         try:
-            filesum = misc.checksum(csum_type, self.localPkg())
+            filesum = misc.checksum(csum_type, self.localPkg(),
+                                    datasize=self.packagesize)
         except Errors.MiscError:
             return False
         
         if filesum != csum:
             return False
         
+        self._verify_local_pkg_cache = nst
+
         return True
         
     def prcoPrintable(self, prcoTuple):
@@ -730,6 +825,214 @@ class YumAvailablePackage(PackageObject, RpmBase):
                 csumid = 0
             self._checksums.append((ctype, csum, csumid))
 
+# from here down this is for dumping a package object back out to metadata
+    
+    
+    def _return_remote_location(self):
+        # break self.remote_url up into smaller pieces
+        base = os.path.dirname(self.remote_url)
+        href = os.path.basename(self.remote_url)
+        msg = """<location xml:base="%s" href="%s"/>\n""" % (
+                  misc.to_xml(base,attrib=True), misc.to_xml(href, attrib=True))
+        return msg
+        
+    def _dump_base_items(self):
+        
+        packager = url = ''
+        if self.packager:
+            packager = misc.to_unicode(misc.to_xml(self.packager))
+        
+        if self.url:
+            url = misc.to_unicode(misc.to_xml(self.url))
+        (csum_type, csum, csumid) = self.checksums[0]
+        msg = """
+  <name>%s</name>
+  <arch>%s</arch>
+  <version epoch="%s" ver="%s" rel="%s"/>
+  <checksum type="%s" pkgid="YES">%s</checksum>
+  <summary>%s</summary>
+  <description>%s</description>
+  <packager>%s</packager>
+  <url>%s</url>
+  <time file="%s" build="%s"/>
+  <size package="%s" installed="%s" archive="%s"/>\n""" % (self.name, 
+         self.arch, self.epoch, self.ver, self.rel, csum_type, csum, 
+         misc.to_unicode(misc.to_xml(self.summary)), 
+         misc.to_unicode(misc.to_xml(self.description)), 
+         packager, url, self.filetime, 
+         self.buildtime, self.packagesize, self.size, self.archivesize)
+        
+        msg += self._return_remote_location()
+        return msg
+
+    def _dump_format_items(self):
+        msg = "  <format>\n"
+        if self.license:
+            msg += """    <rpm:license>%s</rpm:license>\n""" % misc.to_xml(self.license)
+        else:
+            msg += """    <rpm:license/>\n"""
+            
+        if self.vendor:
+            msg += """    <rpm:vendor>%s</rpm:vendor>\n""" % misc.to_xml(self.vendor)
+        else:
+            msg += """    <rpm:vendor/>\n"""
+            
+        if self.group:
+            msg += """    <rpm:group>%s</rpm:group>\n""" % misc.to_xml(self.group)
+        else:
+            msg += """    <rpm:group/>\n"""
+            
+        if self.buildhost:
+            msg += """    <rpm:buildhost>%s</rpm:buildhost>\n""" % misc.to_xml(self.buildhost)
+        else:
+            msg += """    <rpm:buildhost/>\n"""
+            
+        if self.sourcerpm:
+            msg += """    <rpm:sourcerpm>%s</rpm:sourcerpm>\n""" % misc.to_xml(self.sourcerpm)
+        msg +="""    <rpm:header-range start="%s" end="%s"/>""" % (self.hdrstart,
+                                                               self.hdrend)
+        msg += self._dump_pco('provides')
+        msg += self._dump_requires()
+        msg += self._dump_pco('conflicts')         
+        msg += self._dump_pco('obsoletes')         
+        msg += self._dump_files(True)
+        msg += """\n  </format>"""
+        return msg
+
+    def _dump_pco(self, pcotype):
+           
+        msg = ""
+        mylist = getattr(self, pcotype)
+        if mylist: msg = "\n    <rpm:%s>\n" % pcotype
+        for (name, flags, (e,v,r)) in mylist:
+            pcostring = '''      <rpm:entry name="%s"''' % misc.to_xml(name, attrib=True)
+            if flags:
+                pcostring += ''' flags="%s"''' % flags
+                if e:
+                    pcostring += ''' epoch="%s"''' % e
+                if v:
+                    pcostring += ''' ver="%s"''' % v
+                if r:
+                    pcostring += ''' rel="%s"''' % r
+                    
+            pcostring += "/>\n"
+            msg += pcostring
+            
+        if mylist: msg += "    </rpm:%s>" % pcotype
+        return msg
+    
+    def _return_primary_files(self, list_of_files=None):
+        returns = {}
+        if list_of_files is None:
+            list_of_files = self.returnFileEntries('file')
+        for item in list_of_files:
+            if item is None:
+                continue
+            if misc.re_primary_filename(item):
+                returns[item] = 1
+        return returns.keys()
+
+    def _return_primary_dirs(self):
+        returns = {}
+        for item in self.returnFileEntries('dir'):
+            if item is None:
+                continue
+            if misc.re_primary_dirname(item):
+                returns[item] = 1
+        return returns.keys()
+        
+        
+    def _dump_files(self, primary=False):
+        msg =""
+        if not primary:
+            files = self.returnFileEntries('file')
+            dirs = self.returnFileEntries('dir')
+            ghosts = self.returnFileEntries('ghost')
+        else:
+            files = self._return_primary_files()
+            ghosts = self._return_primary_files(list_of_files = self.returnFileEntries('ghost'))
+            dirs = self._return_primary_dirs()
+                
+        for fn in files:
+            msg += """    <file>%s</file>\n""" % misc.to_xml(fn)
+        for fn in dirs:
+            msg += """    <file type="dir">%s</file>\n""" % misc.to_xml(fn)
+        for fn in ghosts:
+            msg += """    <file type="ghost">%s</file>\n""" % misc.to_xml(fn)
+        
+        return msg
+
+
+    def _requires_with_pre(self):
+        raise NotImplementedError()
+                    
+    def _dump_requires(self):
+        """returns deps in format"""
+        mylist = self._requires_with_pre()
+
+        msg = ""
+
+        if mylist: msg = "\n    <rpm:requires>\n"
+        for (name, flags, (e,v,r),pre) in mylist:
+            if name.startswith('rpmlib('):
+                continue
+            prcostring = '''      <rpm:entry name="%s"''' % misc.to_xml(name, attrib=True)
+            if flags:
+                prcostring += ''' flags="%s"''' % flags
+                if e:
+                    prcostring += ''' epoch="%s"''' % e
+                if v:
+                    prcostring += ''' ver="%s"''' % v
+                if r:
+                    prcostring += ''' rel="%s"''' % r
+            if pre:
+                prcostring += ''' pre="%s"''' % pre
+                    
+            prcostring += "/>\n"
+            msg += prcostring
+            
+        if mylist: msg += "    </rpm:requires>"
+        return msg
+
+    def _dump_changelog(self, clog_limit):
+        if not self.changelog:
+            return ""
+        msg = "\n"
+        # We need to output them "backwards", so the oldest is first
+        if not clog_limit:
+            clogs = self.changelog
+        else:
+            clogs = self.changelog[:clog_limit]
+        for (ts, author, content) in reversed(clogs):
+            msg += """<changelog author="%s" date="%s">%s</changelog>\n""" % (
+                        misc.to_xml(author, attrib=True), misc.to_xml(str(ts)), 
+                        misc.to_xml(content))
+        return msg
+
+    def xml_dump_primary_metadata(self):
+        msg = """\n<package type="rpm">"""
+        msg += misc.to_unicode(self._dump_base_items())
+        msg += misc.to_unicode(self._dump_format_items())
+        msg += """\n</package>"""
+        return misc.to_utf8(msg)
+
+    def xml_dump_filelists_metadata(self):
+        msg = """\n<package pkgid="%s" name="%s" arch="%s">
+    <version epoch="%s" ver="%s" rel="%s"/>\n""" % (self.checksum, self.name, 
+                                     self.arch, self.epoch, self.ver, self.rel)
+        msg += misc.to_unicode(self._dump_files())
+        msg += "</package>\n"
+        return misc.to_utf8(msg)
+
+    def xml_dump_other_metadata(self, clog_limit=0):
+        msg = """\n<package pkgid="%s" name="%s" arch="%s">
+    <version epoch="%s" ver="%s" rel="%s"/>\n""" % (self.checksum, self.name, 
+                                     self.arch, self.epoch, self.ver, self.rel)
+        msg += "%s\n</package>\n" % misc.to_unicode(self._dump_changelog(clog_limit))
+        return misc.to_utf8(msg)
+
+
+
 
 class YumHeaderPackage(YumAvailablePackage):
     """Package object built from an rpm header"""
@@ -740,15 +1043,21 @@ class YumHeaderPackage(YumAvailablePackage):
 
         self.hdr = hdr
         self.name = misc.share_data(self.hdr['name'])
-        self.arch = misc.share_data(self.hdr['arch'])
+        this_a = self.hdr['arch']
+        if not this_a: # this should only happen on gpgkeys and other "odd" pkgs
+            this_a = 'noarch'
+        self.arch = misc.share_data(this_a)
         self.epoch = misc.share_data(self.doepoch())
         self.version = misc.share_data(self.hdr['version'])
         self.release = misc.share_data(self.hdr['release'])
         self.ver = self.version
         self.rel = self.release
         self.pkgtup = (self.name, self.arch, self.epoch, self.version, self.release)
-        self.summary = misc.share_data(self.hdr['summary'].replace('\n', ''))
-        self.description = misc.share_data(self.hdr['description'])
+        # Summaries "can be" empty, which rpm return [], see BZ 473239, *sigh*
+        self.summary = self.hdr['summary'] or ''
+        self.summary = misc.share_data(self.summary.replace('\n', ''))
+        self.description = self.hdr['description'] or ''
+        self.description = misc.share_data(self.description)
         self.pkgid = self.hdr[rpm.RPMTAG_SHA1HEADER]
         if not self.pkgid:
             self.pkgid = "%s.%s" %(self.hdr['name'], self.hdr['buildtime'])
@@ -811,6 +1120,9 @@ class YumHeaderPackage(YumAvailablePackage):
     def __getattr__(self, thing):
         #FIXME - if an error - return AttributeError, not KeyError 
         # ONLY FIX THIS AFTER THE API BREAK
+        if thing.startswith('__') and thing.endswith('__'):
+            if not hasattr(self, thing):
+                raise AttributeError, "%s has no attribute %s" % (self, thing)
         return self.hdr[thing]
 
     def doepoch(self):
@@ -872,14 +1184,43 @@ class YumHeaderPackage(YumAvailablePackage):
         # then create a _loadChangelog() method to put them into the 
         # self._changelog attr
         if len(self.hdr['changelogname']) > 0:
-            return zip(self.hdr['changelogtime'],
-                       self.hdr['changelogname'],
-                       self.hdr['changelogtext'])
+            return zip(misc.to_unicode(self.hdr['changelogtime'], errors='replace'),
+                       misc.to_unicode(self.hdr['changelogname'], errors='replace'),
+                       misc.to_unicode(self.hdr['changelogtext'], errors='replace'))
         return []
 
     def returnChecksums(self):
         raise NotImplementedError()
-        
+
+    def _size(self):
+        return self.hdr['size']
+
+    def _is_pre_req(self, flag):
+        """check the flags for a requirement, return 1 or 0 whether or not requires
+           is a pre-requires or a not"""
+        # FIXME this should probably be put in rpmUtils.miscutils since 
+        # - that's what it is
+        if flag is not None:
+            # Note: RPMSENSE_PREREQ == 0 since rpm-4.4'ish
+            if flag & (rpm.RPMSENSE_PREREQ |
+                       rpm.RPMSENSE_SCRIPT_PRE |
+                       rpm.RPMSENSE_SCRIPT_POST):
+                return 1
+        return 0
+
+    def _requires_with_pre(self):
+        """returns requires with pre-require bit"""
+        name = self.hdr[rpm.RPMTAG_REQUIRENAME]
+        lst = self.hdr[rpm.RPMTAG_REQUIREFLAGS]
+        flag = map(flagToString, lst)
+        pre = map(self._is_pre_req, lst)
+        lst = self.hdr[rpm.RPMTAG_REQUIREVERSION]
+        vers = map(stringToVersion, lst)
+        if name is not None:
+            lst = zip(name, flag, vers, pre)
+        mylist = misc.unique(lst)
+        return mylist
+
 class _CountedReadFile:
     """ Has just a read() method, and keeps a count so we can find out how much
         has been read. Implemented so we can get the real size of the file from
@@ -926,7 +1267,7 @@ class _PkgVerifyProb:
         return ret
 
 # From: lib/rpmvf.h ... not in rpm *sigh*
-_RPMVERIFY_MD5      = (1 << 0)
+_RPMVERIFY_DIGEST     = (1 << 0)
 _RPMVERIFY_FILESIZE = (1 << 1)
 _RPMVERIFY_LINKTO   = (1 << 2)
 _RPMVERIFY_USER     = (1 << 3)
@@ -939,10 +1280,11 @@ _installed_repo = FakeRepository('installed')
 _installed_repo.cost = 0
 class YumInstalledPackage(YumHeaderPackage):
     """super class for dealing with packages in the rpmdb"""
-    def __init__(self, hdr):
-        fakerepo = _installed_repo
-        YumHeaderPackage.__init__(self, fakerepo, hdr)
-        
+    def __init__(self, hdr, yumdb=None):
+        YumHeaderPackage.__init__(self, _installed_repo, hdr)
+        if yumdb:
+            self.yumdb_info = yumdb.get_package(self)
+
     def verify(self, patterns=[], deps=False, script=False,
                fake_problems=True, all=False):
         """verify that the installed files match the packaged checksum
@@ -970,9 +1312,18 @@ class YumInstalledPackage(YumHeaderPackage):
         prelink_cmd = "/usr/sbin/prelink"
         have_prelink = os.path.exists(prelink_cmd)
 
+        # determine what checksum algo to use:
+        csum_type = 'md5' # default for legacy
+        if hasattr(rpm, 'RPMTAG_FILEDIGESTALGO'):
+            csum_num = self.hdr[rpm.RPMTAG_FILEDIGESTALGO]
+            if csum_num:
+                if csum_num in RPM_CHECKSUM_TYPES:
+                    csum_type = RPM_CHECKSUM_TYPES[csum_num]
+                # maybe an else with an error code here? or even a verify issue?
+
         for filetuple in fi:
             #tuple is: (filename, fsize, mode, mtime, flags, frdev?, inode, link,
-            #           state, vflags?, user, group, md5sum(or none for dirs) 
+            #           state, vflags?, user, group, checksum(or none for dirs) 
             (fn, size, mode, mtime, flags, dev, inode, link, state, vflags, 
                        user, group, csum) = filetuple
             if patterns:
@@ -1112,15 +1463,15 @@ class YumInstalledPackage(YumHeaderPackage):
                 # just so we get the size correct.
                 if (check_content and
                     ((have_prelink and vflags & _RPMVERIFY_FILESIZE) or
-                     (csum and vflags & _RPMVERIFY_MD5))):
+                     (csum and vflags & _RPMVERIFY_DIGEST))):
                     try:
-                        my_csum = misc.checksum('md5', fn)
+                        my_csum = misc.checksum(csum_type, fn)
                         gen_csum = True
                     except Errors.MiscError:
                         # Don't have permission?
                         gen_csum = False
 
-                    if csum and vflags & _RPMVERIFY_MD5 and not gen_csum:
+                    if csum and vflags & _RPMVERIFY_DIGEST and not gen_csum:
                         prob = _PkgVerifyProb('genchecksum',
                                               'checksum not available', ftypes)
                         prob.database_value = csum
@@ -1133,10 +1484,10 @@ class YumInstalledPackage(YumHeaderPackage):
                         (ig, fp,er) = os.popen3([prelink_cmd, "-y", fn])
                         # er.read(1024 * 1024) # Try and get most of the stderr
                         fp = _CountedReadFile(fp)
-                        my_csum = misc.checksum('md5', fp)
+                        my_csum = misc.checksum(csum_type, fp)
                         my_st_size = fp.read_size
 
-                    if (csum and vflags & _RPMVERIFY_MD5 and gen_csum and
+                    if (csum and vflags & _RPMVERIFY_DIGEST and gen_csum and
                         my_csum != csum):
                         prob = _PkgVerifyProb('checksum',
                                               'checksum does not match', ftypes)
@@ -1206,10 +1557,20 @@ class YumLocalPackage(YumHeaderPackage):
         YumHeaderPackage.__init__(self, fakerepo, hdr)
         self.id = self.pkgid
         self._stat = os.stat(self.localpath)
-        self.filetime = str(self._stat[-1])
+        self.filetime = str(self._stat[-2])
         self.packagesize = str(self._stat[6])
         self.arch = self.isSrpm()
         self.pkgtup = (self.name, self.arch, self.epoch, self.ver, self.rel)
+        self._hdrstart = None
+        self._hdrend = None
+        self.arch = self.isSrpm()
+        self.checksum_type = misc._default_checksums[0]
+
+        # these can be set by callers that need these features (ex: createrepo)
+        self._reldir = None 
+        self._baseurl = "" 
+        # self._packagenumber will be needed when we do sqlite creation here
+
         
     def isSrpm(self):
         if self.tagByName('sourcepackage') == 1 or not self.tagByName('sourcerpm'):
@@ -1220,12 +1581,92 @@ class YumLocalPackage(YumHeaderPackage):
     def localPkg(self):
         return self.localpath
     
-    def _do_checksum(self, checksum_type='sha'):
+    def _do_checksum(self, checksum_type=None):
+        if checksum_type is None:
+            checksum_type = misc._default_checksums[0]
         if not self._checksum:
             self._checksum = misc.checksum(checksum_type, self.localpath)
-            
+            self._checksums = [(checksum_type, self._checksum, 1)]
+
         return self._checksum    
 
-    checksum = property(fget=lambda self: self._do_checksum())    
-    
+    checksum = property(fget=lambda self: self._do_checksum())   
+
+    def returnChecksums(self):
+        self._do_checksum()
+        return self._checksums
+
+    def verifyLocalPkg(self):
+        """ don't bother "checking" the package matches itself. """
+        return True
+
+    def _get_header_byte_range(self):
+        """takes an rpm file or fileobject and returns byteranges for location of the header"""
+        if self._hdrstart and self._hdrend:
+            return (self._hdrstart, self._hdrend)
+      
+           
+        fo = open(self.localpath, 'r')
+        #read in past lead and first 8 bytes of sig header
+        fo.seek(104)
+        # 104 bytes in
+        binindex = fo.read(4)
+        # 108 bytes in
+        (sigindex, ) = struct.unpack('>I', binindex)
+        bindata = fo.read(4)
+        # 112 bytes in
+        (sigdata, ) = struct.unpack('>I', bindata)
+        # each index is 4 32bit segments - so each is 16 bytes
+        sigindexsize = sigindex * 16
+        sigsize = sigdata + sigindexsize
+        # we have to round off to the next 8 byte boundary
+        disttoboundary = (sigsize % 8)
+        if disttoboundary != 0:
+            disttoboundary = 8 - disttoboundary
+        # 112 bytes - 96 == lead, 8 = magic and reserved, 8 == sig header data
+        hdrstart = 112 + sigsize  + disttoboundary
+        
+        fo.seek(hdrstart) # go to the start of the header
+        fo.seek(8,1) # read past the magic number and reserved bytes
+
+        binindex = fo.read(4) 
+        (hdrindex, ) = struct.unpack('>I', binindex)
+        bindata = fo.read(4)
+        (hdrdata, ) = struct.unpack('>I', bindata)
+        
+        # each index is 4 32bit segments - so each is 16 bytes
+        hdrindexsize = hdrindex * 16 
+        # add 16 to the hdrsize to account for the 16 bytes of misc data b/t the
+        # end of the sig and the header.
+        hdrsize = hdrdata + hdrindexsize + 16
+        
+        # header end is hdrstart + hdrsize 
+        hdrend = hdrstart + hdrsize 
+        fo.close()
+        self._hdrstart = hdrstart
+        self._hdrend = hdrend
+       
+        return (hdrstart, hdrend)
+        
+    hdrend = property(fget=lambda self: self._get_header_byte_range()[1])
+    hdrstart = property(fget=lambda self: self._get_header_byte_range()[0])
+
+    def _return_remote_location(self):
+
+        # if we start seeing fullpaths in the location tag - this is the culprit
+        if self._reldir and self.localpath.startswith(self._reldir):
+            relpath = self.localpath.replace(self._reldir, '')
+            if relpath[0] == '/': relpath = relpath[1:]
+        else:
+            relpath = self.localpath
+
+        if self._baseurl:
+            msg = """<location xml:base="%s" href="%s"/>\n""" % (
+                                     misc.to_xml(self._baseurl, attrib=True),
+                                     misc.to_xml(relpath, attrib=True))
+        else:
+            msg = """<location href="%s"/>\n""" % misc.to_xml(relpath, attrib=True)
+
+        return msg
+
 
