@@ -37,6 +37,8 @@ from weakref import proxy as weakref
 
 from yum import _
 
+from yum.i18n import utf8_width
+
 # TODO: expose rpm package sack objects to plugins (once finished)
 # TODO: allow plugins to use the existing config stuff to define options for
 # their own configuration files (would replace confString() etc).
@@ -81,6 +83,7 @@ SLOT_TO_CONDUIT = {
     'config': 'ConfigPluginConduit',
     'postconfig': 'PostConfigPluginConduit',
     'init': 'InitPluginConduit',
+    'args': 'ArgsPluginConduit',
     'predownload': 'DownloadPluginConduit',
     'postdownload': 'DownloadPluginConduit',
     'prereposetup': 'PreRepoSetupPluginConduit',
@@ -115,7 +118,7 @@ class YumPlugins:
     '''
 
     def __init__(self, base, searchpath, optparser=None, types=None, 
-            pluginconfpath=None,disabled=None):
+            pluginconfpath=None,disabled=None,enabled=None):
         '''Initialise the instance.
 
         @param base: The
@@ -138,6 +141,7 @@ class YumPlugins:
         self.cmdline = (None, None)
         self.verbose_logger = logging.getLogger("yum.verbose.YumPlugins")
         self.disabledPlugins = disabled
+        self.enabledPlugins  = enabled
         if types is None:
             types = ALL_TYPES
         if not isinstance(types, (list, tuple)):
@@ -168,8 +172,9 @@ class YumPlugins:
         conduitcls = eval(conduitcls)       # Convert name to class object
 
         for modname, func in self._pluginfuncs[slotname]:
-            self.verbose_logger.debug('Running "%s" handler for "%s" plugin', slotname,
-                modname)
+            self.verbose_logger.log(logginglevels.DEBUG_4,
+                                    'Running "%s" handler for "%s" plugin',
+                                    slotname, modname)
     
             _, conf = self._plugins[modname]
             func(conduitcls(self, self.base, conf, **kwargs))
@@ -186,19 +191,25 @@ class YumPlugins:
 
         # Import plugins 
         self._used_disable_plugin = set()
+        self._used_enable_plugin  = set()
         for dir in self.searchpath:
             if not os.path.isdir(dir):
                 continue
-            for modulefile in glob.glob('%s/*.py' % dir):
+            for modulefile in sorted(glob.glob('%s/*.py' % dir)):
                 self._loadplugin(modulefile, types)
 
-        if self._plugins:
+        # If we are in verbose mode we get the full 'Loading "blah" plugin' lines
+        if (self._plugins and
+            not self.verbose_logger.isEnabledFor(logginglevels.DEBUG_3)):
             # Mostly copied from YumOutput._outKeyValFill()
             key = _("Loaded plugins: ")
             val = ", ".join(sorted(self._plugins))
-            nxt = ' ' * (len(key) - 2) + ': '
+            nxt = ' ' * (utf8_width(key) - 2) + ': '
+            width = 80
+            if hasattr(self.base, 'term'):
+                width = self.base.term.columns
             self.verbose_logger.log(logginglevels.INFO_2,
-                                    fill(val, width=80, initial_indent=key,
+                                    fill(val, width=width, initial_indent=key,
                                          subsequent_indent=nxt))
 
         if self.disabledPlugins:
@@ -207,6 +218,26 @@ class YumPlugins:
                     self.verbose_logger.log(logginglevels.INFO_2,
                                             _("No plugin match for: %s") % wc)
         del self._used_disable_plugin
+        if self.enabledPlugins:
+            for wc in self.enabledPlugins:
+                if wc not in self._used_enable_plugin:
+                    self.verbose_logger.log(logginglevels.INFO_2,
+                                            _("No plugin match for: %s") % wc)
+        del self._used_enable_plugin
+
+    @staticmethod
+    def _plugin_cmdline_match(modname, plugins, used):
+        """ Check if this plugin has been temporary enabled/disabled. """
+        if plugins is None:
+            return False
+
+        for wc in plugins:
+            if fnmatch.fnmatch(modname, wc):
+                used.add(wc)
+                return True
+
+        return False
+
 
     def _loadplugin(self, modulefile, types):
         '''Attempt to import a plugin module and register the hook methods it
@@ -216,29 +247,41 @@ class YumPlugins:
         modname = modname.split('.py')[0]
 
         conf = self._getpluginconf(modname)
-        if not conf or not config.getOption(conf, 'main', 'enabled', 
-                config.BoolOption(False)):
-            self.verbose_logger.debug(_('"%s" plugin is disabled'), modname)
+        if (not conf or
+            (not config.getOption(conf, 'main', 'enabled',
+                                  config.BoolOption(False)) and
+             not self._plugin_cmdline_match(modname, self.enabledPlugins,
+                                            self._used_enable_plugin))):
+            self.verbose_logger.debug(_('Not loading "%s" plugin, as it is disabled'), modname)
             return
 
-        fp, pathname, description = imp.find_module(modname, [dir])
         try:
-            module = imp.load_module(modname, fp, pathname, description)
-        finally:
-            fp.close()
+            fp, pathname, description = imp.find_module(modname, [dir])
+            try:
+                module = imp.load_module(modname, fp, pathname, description)
+            finally:
+                fp.close()
+        except:
+            if self.verbose_logger.isEnabledFor(logginglevels.DEBUG_4):
+                raise # Give full backtrace:
+            self.verbose_logger.error(_('Plugin "%s" can\'t be imported') %
+                                      modname)
+            return
 
         # Check API version required by the plugin
         if not hasattr(module, 'requires_api_version'):
-             raise Errors.ConfigError(
-                _('Plugin "%s" doesn\'t specify required API version') % modname
-                )
+            self.verbose_logger.error(
+                _('Plugin "%s" doesn\'t specify required API version') %
+                modname)
+            return
         if not apiverok(API_VERSION, module.requires_api_version):
-            raise Errors.ConfigError(
+            self.verbose_logger.error(
                 _('Plugin "%s" requires API %s. Supported API is %s.') % (
                     modname,
                     module.requires_api_version,
                     API_VERSION,
                     ))
+            return
 
         # Check plugin type against filter
         plugintypes = getattr(module, 'plugin_type', ALL_TYPES)
@@ -256,12 +299,15 @@ class YumPlugins:
 
             if plugintype not in types:
                 return
-        # Check if this plugin has been temporary disabled
-        if self.disabledPlugins:
-            for wc in self.disabledPlugins:
-                if fnmatch.fnmatch(modname, wc):
-                    self._used_disable_plugin.add(wc)
-                    return
+
+        #  This should really work like enable/disable repo. and be based on the
+        # cmd line order ... but the API doesn't really allow that easily.
+        # FIXME: Fix for 4.*
+        if (self._plugin_cmdline_match(modname, self.disabledPlugins,
+                                       self._used_disable_plugin) and
+            not self._plugin_cmdline_match(modname, self.enabledPlugins,
+                                           self._used_enable_plugin)):
+            return
 
         self.verbose_logger.log(logginglevels.DEBUG_3, _('Loading "%s" plugin'),
                                 modname)
@@ -365,6 +411,9 @@ class PluginConduit:
         @return: the global optparse.OptionParser instance used by Yum. May be
             None if an OptionParser isn't in use.
         '''
+        # ' xemacs highlighting hack
+        # This isn't API compatible :(
+        # return self._parent.optparser.plugin_option_group
         return self._parent.optparser
 
     def confString(self, section, opt, default=None):
@@ -375,6 +424,7 @@ class PluginConduit:
         @param default: Value to read if option is missing.
         @return: String option value read, or default if option was missing.
         '''
+        # ' xemacs highlighting hack
         return config.getOption(self._conf, section, opt, config.Option(default))
 
     def confInt(self, section, opt, default=None):
@@ -466,6 +516,15 @@ class InitPluginConduit(PluginConduit):
         '''
         return self._base.repos
 
+class ArgsPluginConduit(InitPluginConduit):
+
+    def __init__(self, parent, base, conf, args):
+        InitPluginConduit.__init__(self, parent, base, conf)
+        self._args = args
+
+    def getArgs(self):
+        return self._args
+
 class PreRepoSetupPluginConduit(InitPluginConduit):
 
     def getCmdLine(self):
