@@ -16,25 +16,28 @@
 import rpm
 import types
 import warnings
+import glob
+import os
+import os.path
 
 from rpmUtils import miscutils
 from rpmUtils.transaction import initReadOnlyTransaction
 import misc
 import Errors
-from packages import YumInstalledPackage
+from packages import YumInstalledPackage, parsePackages
 from packageSack import PackageSackBase
 
 # For returnPackages(patterns=)
 import fnmatch
 import re
 
-from misc import to_unicode
+from yum.i18n import to_unicode
 import constants
 
 class RPMInstalledPackage(YumInstalledPackage):
 
     def __init__(self, rpmhdr, index, rpmdb):
-        YumInstalledPackage.__init__(self, rpmhdr)
+        YumInstalledPackage.__init__(self, rpmhdr, yumdb=rpmdb.yumdb)
         # NOTE: We keep summary/description/url because it doesn't add much
         # and "yum search" uses them all.
         self.url       = rpmhdr['url']
@@ -54,14 +57,21 @@ class RPMInstalledPackage(YumInstalledPackage):
 
         ts = self.rpmdb.readOnlyTS()
         mi = ts.dbMatch(0, self.idx)
-        return mi.next()
-        return self.hdr
+        try:
+            return mi.next()
+        except StopIteration:
+            raise Errors.PackageSackError, 'Rpmdb changed underneath us'
 
     def __getattr__(self, varname):
         self.hdr = val = self._get_hdr()
         self._has_hdr = True
-        if varname != 'hdr':   # This is very unusual, for anything it does
-            val = val[varname] # happen for it might be worth adding at __init_
+        if varname.startswith('__') and varname.endswith('__'):
+            if not hasattr(self, varname):
+                raise AttributeError, "%s has no attribute %s" % (self, varname)
+            
+        if varname != 'hdr':   #  This is unusual, for anything that happens
+            val = val[varname] # a lot we should preload at __init__.
+                               # Also note that pkg.no_value raises KeyError.
 
         return val
 
@@ -96,6 +106,9 @@ class RPMDBPackageSack(PackageSackBase):
         self._get_pro_cache = {}
         self._get_req_cache  = {}
         self.ts = None
+        self.auto_close = False # this forces a self.ts.close() after
+                                     # most operations so it doesn't leave
+                                     # any lingering locks.
 
         self._cache = {
             'provides' : { },
@@ -104,6 +117,9 @@ class RPMDBPackageSack(PackageSackBase):
             'obsoletes' : { },
             }
         
+        addldb_path = os.path.normpath(root + '/' + '/var/lib/yum/yumdb')
+        self.yumdb = RPMDBAdditionalData(db_path=addldb_path)
+
     def _get_pkglist(self):
         '''Getter for the pkglist property. 
         Returns a list of package tuples.
@@ -135,6 +151,8 @@ class RPMDBPackageSack(PackageSackBase):
     def readOnlyTS(self):
         if not self.ts:
             self.ts =  initReadOnlyTransaction(root=self.root)
+        if not self.ts.open:
+            self.ts = initReadOnlyTransaction(root=self.root)
         return self.ts
 
     def buildIndexes(self):
@@ -168,6 +186,9 @@ class RPMDBPackageSack(PackageSackBase):
             if not result.has_key(pkg.pkgid):
                 result[pkg.pkgid] = pkg
         
+        if self.auto_close:
+            self.ts.close()
+
         return result.values()
 
     def searchFiles(self, name):
@@ -184,6 +205,10 @@ class RPMDBPackageSack(PackageSackBase):
         del mi
 
         result = result.values()
+
+        if self.auto_close:
+            self.ts.close()
+
         return result
         
     def searchPrco(self, name, prcotype):
@@ -195,7 +220,7 @@ class RPMDBPackageSack(PackageSackBase):
         ts = self.readOnlyTS()
         result = {}
         tag = self.DEP_TABLE[prcotype][0]
-        mi = ts.dbMatch(tag, name)
+        mi = ts.dbMatch(tag, misc.to_utf8(name))
         for hdr in mi:
             po = self._makePackageObject(hdr, mi.instance())
             result[po.pkgid] = po
@@ -209,6 +234,10 @@ class RPMDBPackageSack(PackageSackBase):
         
         result = result.values()
         self._cache[prcotype][name] = result
+
+        if self.auto_close:
+            self.ts.close()
+
         return result
 
     def searchProvides(self, name):
@@ -296,7 +325,12 @@ class RPMDBPackageSack(PackageSackBase):
                 if self._match_repattern(rpats, hdr):
                     self._makePackageObject(hdr, idx)
             self._completely_loaded = patterns is None
-        return self._idx2pkg.values()
+
+        pkgobjlist = self._idx2pkg.values()
+        if patterns:
+            pkgobjlist = parsePackages(pkgobjlist, patterns, not ignore_case)
+            pkgobjlist = pkgobjlist[0] + pkgobjlist[1]
+        return pkgobjlist
 
     @staticmethod
     def _find_search_fields(fields, searchstrings, hdr):
@@ -358,6 +392,9 @@ class RPMDBPackageSack(PackageSackBase):
             if hdr['name'] != 'gpg-pubkey':
                 yield (hdr, mi.instance())
         del mi
+        if self.auto_close:
+            self.ts.close()
+
 
     def _header_from_index(self, idx):
         """returns a package header having been given an index"""
@@ -377,6 +414,9 @@ class RPMDBPackageSack(PackageSackBase):
             del hdr
 
         del mi
+        if self.auto_close:
+            self.ts.close()
+
 
     def _search(self, name=None, epoch=None, ver=None, rel=None, arch=None):
         '''Generator that yields matching packages
@@ -417,6 +457,10 @@ class RPMDBPackageSack(PackageSackBase):
                     break
             else:
                 ret.append(po)
+
+        if self.auto_close:
+            self.ts.close()
+
         return ret
 
     def _makePackageObject(self, hdr, index):
@@ -536,7 +580,7 @@ class RPMDBPackageSack(PackageSackBase):
 
     def getProvides(self, name, flags=None, version=(None, None, None)):
         """searches the rpmdb for what provides the arguments
-           returns a list of pkgtuples of providing packages, possibly empty"""
+           returns a list of pkg objects of providing packages, possibly empty"""
 
         name = misc.share_data(name)
         deptup = self._genDeptup(name, flags, version)
@@ -590,7 +634,179 @@ class RPMDBPackageSack(PackageSackBase):
     def whatRequires(self, name, flags, version):
         # XXX deprecate?
         return [po.pkgtup for po in self.getRequires(name, flags, version)]
-            
+
+def _sanitize(path):
+    return path.replace('/', '').replace('~', '')
+
+
+class RPMDBAdditionalData(object):
+    """class for access to the additional data not able to be stored in the
+       rpmdb"""
+    # dir: /var/lib/yum/yumdb/
+    # pkgs stored in name[0]/name[1]/pkgid-name-ver-rel-arch dirs
+    # dirs have files per piece of info we're keeping
+    #    repoid, install reason, status, blah, (group installed for?), notes?
+    
+    def __init__(self, db_path='/var/lib/yum/yumdb'):
+        self.conf = misc.GenericHolder()
+        self.conf.db_path = db_path
+        self.conf.writable = False
+        
+        self._packages = {} # pkgid = dir
+        if not os.path.exists(self.conf.db_path):
+            try:
+                os.makedirs(self.conf.db_path)
+            except (IOError, OSError), e:
+                # some sort of useful thing here? A warning?
+                return
+            self.conf.writable = True
+        else:
+            if os.access(self.conf.db_path, os.W_OK):
+                self.conf.writable = True
+                
+        # glob the path and get a dict of pkgs to their subdir
+        glb = '%s/*/*/' % self.conf.db_path
+        pkgdirs = glob.glob(glb)
+        for d in pkgdirs:
+            pkgid = os.path.basename(d).split('-')[0]
+            self._packages[pkgid] = d
+
+    def _get_dir_name(self, pkgtup, pkgid):
+        if pkgid in self._packages:
+            return self._packages[pkgid]
+        (n, a, e, v,r) = pkgtup
+        n = _sanitize(n) # Please die in a fire rpmbuild
+        thisdir = '%s/%s/%s-%s-%s-%s-%s' % (self.conf.db_path,
+                                            n[0], pkgid, n, v, r, a)
+        self._packages[pkgid] = thisdir
+        return thisdir
+
+    def get_package(self, po=None, pkgtup=None, pkgid=None):
+        """Return an RPMDBAdditionalDataPackage Object for this package"""
+        if po:
+            thisdir = self._get_dir_name(po.pkgtup, po.pkgid)
+        elif pkgtup and pkgid:
+            thisdir = self._get_dir_name(pkgtup, pkgid)
+        else:
+            raise ValueError,"Pass something to RPMDBAdditionalData.get_package"
+        
+        return RPMDBAdditionalDataPackage(self.conf, thisdir)
+
+    def sync_with_rpmdb(self, rpmdbobj):
+        """populate out the dirs and remove all the items no longer in the rpmd
+           and/or populate various bits to the currently installed version"""
+        # TODO:
+        # get list of all items in the yumdb
+        # remove any no longer in the rpmdb/andor migrate them up to the currently
+        # installed version
+        # add entries for items in the rpmdb if they don't exist in the yumdb
+
+        pass
+
+class RPMDBAdditionalDataPackage(object):
+    def __init__(self, conf, pkgdir):
+        self._conf = conf
+        self._mydir = pkgdir
+        # FIXME needs some intelligent caching beyond the FS cache
+        self._read_cached_data = {}
+
+    def _write(self, attr, value):
+        # check for self._conf.writable before going on?
+        if not os.path.exists(self._mydir):
+            os.makedirs(self._mydir)
+
+        attr = _sanitize(attr)
+        if attr in self._read_cached_data:
+            del self._read_cached_data[attr]
+        fn = self._mydir + '/' + attr
+        fn = os.path.normpath(fn)
+        fo = open(fn + '.tmp', 'w')
+        try:
+            fo.write(value)
+        except (OSError, IOError), e:
+            raise AttributeError, "Cannot set attribute %s on %s" % (attr, self)
+
+        fo.flush()
+        fo.close()
+        del fo
+        os.rename(fn +  '.tmp', fn) # even works on ext4 now!:o
+        self._read_cached_data[attr] = value
+    
+    def _read(self, attr):
+        attr = _sanitize(attr)
+
+        if attr in self._read_cached_data:
+            return self._read_cached_data[attr]
+
+        fn = self._mydir + '/' + attr
+        if not os.path.exists(fn):
+            raise AttributeError, "%s has no attribute %s" % (self, attr)
+
+        fo = open(fn, 'r')
+        self._read_cached_data[attr] = fo.read()
+        fo.close()
+        del fo
+        return self._read_cached_data[attr]
+    
+    def _delete(self, attr):
+        """remove the attribute file"""
+
+        self.get(attr)
+        fn = self._mydir + '/' + attr
+        if attr in self._read_cached_data:
+            del self._read_cached_data[attr]
+        if os.path.exists(fn):
+            try:
+                os.unlink(fn)
+            except (IOError, OSError):
+                raise AttributeError, "Cannot delete attribute %s on " % (attr, self)
+    
+    def __getattr__(self, attr):
+        return self._read(attr)
+
+    def __setattr__(self, attr, value):
+        if not attr.startswith('_'):
+            self._write(attr, value)
+        else:
+            object.__setattr__(self, attr, value)
+
+    def __delattr__(self, attr):
+        if not attr.startswith('_'):
+            self._delete(attr)
+        else:
+            object.__delattr__(self, attr)
+
+    def __iter__(self):
+        for item in self._read_cached_data:
+            yield item
+        for item in glob.glob(self._mydir + '/*'):
+            item = item[(len(self._mydir) + 1):]
+            if item in self._read_cached_data:
+                continue
+            yield item
+
+    def clean(self):
+        # purge out everything
+        for item in self:
+            self._delete(item)
+        try:
+            os.rmdir(self._mydir)
+        except OSError:
+            pass
+
+#    def __dir__(self): # for 2.6 and beyond, apparently
+#        return list(self.__iter__()) + self.__dict__.keys()
+
+    def get(self, attr, default=None):
+        """retrieve an add'l data obj"""
+
+        try:
+            res = self._read(attr)
+        except AttributeError:
+            return default
+        return res
+        
+        
 def main():
     sack = RPMDBPackageSack('/')
     for p in sack.simplePkgList():
