@@ -33,8 +33,8 @@ import rpmUtils.miscutils
 import sqlutils
 import constants
 import operator
-import time
-from yum.misc import seq_max_split, to_utf8
+from yum.misc import seq_max_split
+from yum.i18n import to_utf8, to_unicode
 import sys
 
 def catchSqliteException(func):
@@ -46,7 +46,10 @@ def catchSqliteException(func):
             # 2.4.x requires this, but 2.6.x complains about even hasattr()
             # of e.message ... *sigh*
             if sys.hexversion < 0x02050000:
-                raise Errors.RepoError, str(e.message)
+                if hasattr(e,'message'):
+                    raise Errors.RepoError, str(e.message)
+                else:
+                    raise Errors.RepoError, str(e)
             raise Errors.RepoError, str(e)
 
     newFunc.__name__ = func.__name__
@@ -68,6 +71,7 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
         self.repo = repo
         self.state = None
         self._loadedfiles = False
+        self._files = None
         self._read_db_obj(db_obj)
         # for stupid metadata created without epochs listed in the version tag
         # die die
@@ -80,6 +84,7 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
 
         self._changelog = None
         self._hash = None
+        
 
     files = property(fget=lambda self: self._loadFiles())
 
@@ -138,6 +143,10 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
                          'license' : 'rpm_license',
                          'checksum_value' : 'pkgId',
                         }
+
+        if varname.startswith('__') and varname.endswith('__'):
+            if not hasattr(self, varname):
+                raise AttributeError, varname
         
         dbname = varname
         if db2simplemap.has_key(varname):
@@ -206,7 +215,8 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
             cur = self._sql_MD('other',
                                "SELECT date, author, changelog " \
                                "FROM   changelog JOIN packages USING(pkgKey) " \
-                               "WHERE  pkgId = ?", (self.pkgId,))
+                               "WHERE  pkgId = ? ORDER BY date DESC",
+                               (self.pkgId,))
             # Check count(pkgId) here, the same way we do in searchFiles()?
             # Failure mode is much less of a problem.
             for ob in cur:
@@ -219,7 +229,7 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
     
         
     def returnIdSum(self):
-            return (self.checksum_type, self.pkgId)
+        return (self.checksum_type, self.pkgId)
     
     def returnChangelog(self):
         self._loadChangelog()
@@ -629,7 +639,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             return []
 
         result = [] # (pkg, num matches)
-        if len(fields) < 1:
+        if not fields or not searchstrings:
             return result
         
         # NOTE: I can't see any reason not to use this all the time, speed
@@ -765,9 +775,11 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
 
     @catchSqliteException
     def _search(self, prcotype, name, flags, version):
+
         if self._skip_all():
             return {}
-
+        
+        name = to_unicode(name)
         if flags == 0:
             flags = None
         if type(version) in (str, type(None), unicode):
@@ -873,19 +885,30 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         return self._search("requires", name, flags, version)
 
     @catchSqliteException
-    def searchNames(self, names):
+    def searchNames(self, names=[]):
         """return a list of packages matching any of the given names. This is 
            only a match on package name, nothing else"""
         
         if self._skip_all():
             return []
-
+        
         returnList = []
+        if hasattr(self, 'pkgobjlist'):
+            names = set(names)
+            for po in self.pkgobjlist:
+                if po.name not in names:
+                    continue
+                if self._pkgExcluded(po):
+                    continue
+                returnList.append(po)
+            return returnList
+
         max_entries = constants.PATTERNS_INDEXED_MAX
         if len(names) > max_entries:
+            returnList = set() # Unique
             for names in seq_max_split(names, max_entries):
-                returnList.extend(self.searchNames(names))
-            return returnList
+                returnList.update(self.searchNames(names))
+            return list(returnList)
 
         pat_sqls = []
         qsql = """select pkgId,pkgKey,name,epoch,version,release,arch
@@ -907,7 +930,8 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         """return list of packages having prcotype name (any evr and flag)"""
         if self._skip_all():
             return []
-
+        
+        name = to_unicode(name)
         glob = True
         querytype = 'glob'
         if not misc.re_glob(name):
@@ -923,7 +947,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         # If it's not a provides or a filename, we are done
         if prcotype != "provides":
             return results
-        if not glob and name[0] != '/':
+        if name[0] != '/' and (not glob or not misc.re_glob(name[0])):
             return results
 
         # If it is a filename, search the primary.xml file info
@@ -1006,7 +1030,6 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
 
     def db2class(self, db, nevra_only=False):
         print 'die die die die die db2class'
-        pass
         class tmpObject:
             pass
         y = tmpObject()
@@ -1131,6 +1154,16 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             pat_max = constants.PATTERNS_INDEXED_MAX
         if len(patterns) > pat_max:
             patterns = []
+        if ignore_case:
+            patterns = self._sql_esc_glob(patterns)
+        else:
+            tmp = []
+            for pat in patterns:
+                if misc.re_glob(pat):
+                    tmp.append((pat, 'glob'))
+                else:
+                    tmp.append((pat, '='))
+            patterns = tmp
 
         for (repo,cache) in self.primarydb.items():
             if (repoid == None or repoid == repo.id):
@@ -1141,16 +1174,12 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
 
                 pat_sqls = []
                 pat_data = []
-                if ignore_case:
-                    patterns = self._sql_esc_glob(patterns)
-                else:
-                    patterns = [(pat, '') for pat in patterns]
-                for (pattern, esc) in patterns:
+                for (pattern, rest) in patterns:
                     for field in fields:
                         if ignore_case:
-                            pat_sqls.append("%s LIKE ?%s" % (field, esc))
+                            pat_sqls.append("%s LIKE ?%s" % (field, rest))
                         else:
-                            pat_sqls.append("%s GLOB ?" % field)
+                            pat_sqls.append("%s %s ?" % (field, rest))
                         pat_data.append(pattern)
                 if pat_sqls:
                     qsql = _FULL_PARSE_QUERY_BEG + " OR ".join(pat_sqls)
@@ -1186,6 +1215,8 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
 
         returnList = []
         for po in pkgobjlist:
+            if repoid is not None and repoid != po.repoid:
+                continue
             if self._pkgExcluded(po):
                 continue
             returnList.append(po)
