@@ -94,8 +94,10 @@ class TransactionData:
             returnlist.extend(self.pkgdict[pkgtup])
         return returnlist
             
+    # The order we resolve things in _matters_, so for sanity sort the list
+    # otherwise .i386 can be different to .x86_64 etc.
     def getUnresolvedMembers(self):
-        return list(self._unresolvedMembers)
+        return list(sorted(self._unresolvedMembers))
 
     def markAsResolved(self, txmbr):
         self._unresolvedMembers.discard(txmbr)
@@ -125,21 +127,20 @@ class TransactionData:
         else:
             return states[0]
             
-
-    
     def matchNaevr(self, name=None, arch=None, epoch=None, ver=None, rel=None):
         """returns the list of packages matching the args above"""
         if name is None:
             txmbrs = self.getMembers()
         else:
             txmbrs = self._namedict.get(name, [])
+            if arch is None and epoch is None and ver is None and rel is None:
+                return txmbrs[:]
 
         result = []
 
         for txmbr in txmbrs:
             (n, a, e, v, r) = txmbr.pkgtup
-            if name is not None and name != n:
-                continue
+            # Name is done above
             if arch is not None and arch != a:
                 continue
             if epoch is not None and epoch != e:
@@ -154,6 +155,10 @@ class TransactionData:
 
     def _isLocalPackage(self, txmember):
         # Is this the right criteria?
+        # FIXME: This is kinda weird, we really want all local pkgs to be in a
+        # special pkgsack before this point ... so that "yum up ./*.rpm" works.
+        #  Also FakePackage() sets it off ... which is confusing and not what
+        # happens IRL.
         return txmember.ts_state in ('u', 'i') and not isinstance(txmember.po, (YumInstalledPackage, YumAvailablePackageSqlite))
 
     def _allowedMultipleInstalls(self, po):
@@ -193,11 +198,12 @@ class TransactionData:
             self.pkgSackPackages += 1
 
         if self.conditionals.has_key(txmember.name):
-            for po in self.conditionals[txmember.name]:
-                if self.rpmdb.contains(po=po):
+            for pkg in self.conditionals[txmember.name]:
+                if self.rpmdb.contains(po=pkg):
                     continue
-                condtxmbr = self.addInstall(po)
-                condtxmbr.setAsDep(po=txmember.po)
+                for condtxmbr in self.install_method(po=pkg):
+                    condtxmbr.setAsDep(po=txmember.po)
+
         self._unresolvedMembers.add(txmember)
 
     def remove(self, pkgtup):
@@ -250,10 +256,11 @@ class TransactionData:
         self.depremoved = []
         self.depinstalled = []
         self.depupdated = []
+        self.failed = []
 
         for txmbr in self.getMembers():
             if txmbr.output_state == TS_UPDATE:
-                if txmbr.isDep and txmbr.reason == "dep":
+                if txmbr.isDep:
                     self.depupdated.append(txmbr)
                 else:
                     self.updated.append(txmbr)
@@ -282,6 +289,8 @@ class TransactionData:
                 
             elif txmbr.output_state == TS_OBSOLETING:
                 self.installed.append(txmbr)
+            elif txmbr.output_state == TS_FAILED:
+                self.failed.append(txmbr)
                 
             else:
                 pass
@@ -295,6 +304,7 @@ class TransactionData:
         self.depremoved.sort()
         self.instgroups.sort()
         self.removedgroups.sort()
+        self.failed.sort()
 
     
     def addInstall(self, po):
@@ -302,7 +312,7 @@ class TransactionData:
            takes a packages object and returns a TransactionMember Object"""
 
         if self._allowedMultipleInstalls(po):
-           return self.addTrueInstall(po)
+            return self.addTrueInstall(po)
     
         txmbr = TransactionMember(po)
         txmbr.current_state = TS_AVAILABLE
@@ -344,7 +354,7 @@ class TransactionData:
            takes a packages object and returns a TransactionMember Object"""
         
         if self._allowedMultipleInstalls(po):
-           return self.addTrueInstall(po)
+            return self.addTrueInstall(po)
             
         txmbr = TransactionMember(po)
         txmbr.current_state = TS_AVAILABLE
@@ -366,7 +376,7 @@ class TransactionData:
         txmbr.current_state = TS_INSTALL
         txmbr.output_state =  TS_UPDATED
         txmbr.po.state = TS_UPDATED
-        txmbr.ts_state = None
+        txmbr.ts_state = None # FIXME: should use a real state here.
         txmbr.relatedto.append((updating_po, 'updatedby'))
         txmbr.updated_by.append(updating_po)
         self.add(txmbr)
@@ -394,7 +404,7 @@ class TransactionData:
         txmbr.current_state = TS_INSTALL
         txmbr.output_state =  TS_OBSOLETED
         txmbr.po.state = TS_OBSOLETED
-        txmbr.ts_state = None
+        txmbr.ts_state = None # FIXME: should use a real state here.
         txmbr.relatedto.append((obsoleting_po, 'obsoletedby'))
         txmbr.obsoleted_by.append(obsoleting_po)
         self.add(txmbr)
@@ -552,10 +562,7 @@ class TransactionMember:
             self.depends_on.append(po)
 
     def __cmp__(self, other):
-        result = cmp(self.name, other.name)
-        if not result:
-            return cmp(id(self), id(other))
-        return result
+        return cmp(self.po, other.po)
 
     def __hash__(self):
         return object.__hash__(self)
