@@ -24,9 +24,11 @@ Update metadata (updateinfo.xml) parsing.
 import sys
 import gzip
 
-from yum.i18n import utf8_text_wrap
+from yum.i18n import utf8_text_wrap, to_utf8
 from yum.yumRepo import YumRepository
+from yum.packages import FakeRepository
 from yum.misc import to_xml
+import Errors
 
 import rpmUtils.miscutils
 
@@ -295,11 +297,16 @@ class UpdateMetadata(object):
     The root update metadata object.
     """
 
-    def __init__(self):
+    def __init__(self, repos=[]):
         self._notices = {}
         self._cache = {}    # a pkg nvr => notice cache for quick lookups
         self._no_cache = {}    # a pkg name only => notice list
         self._repos = []    # list of repo ids that we've parsed
+        for repo in repos:
+            try: # attempt to grab the updateinfo.xml.gz from the repodata
+                self.add(repo)
+            except Errors.RepoMDError:
+                continue # No metadata found for this repo
 
     def get_notices(self, name=None):
         """ Return all notices. """
@@ -361,6 +368,8 @@ class UpdateMetadata(object):
                 if not md:
                     raise UpdateNoticeException()
                 infile = gzip.open(md)
+        elif isinstance(obj, FakeRepository):
+            raise Errors.RepoMDError, "No updateinfo for local pkg"
         else:   # obj is a file object
             infile = obj
 
@@ -377,11 +386,13 @@ class UpdateMetadata(object):
                             no = self._no_cache.setdefault(file['name'], set())
                             no.add(un)
 
-    def __str__(self):
-        ret = ''
+    def __unicode__(self):
+        ret = u''
         for notice in self.notices:
-            ret += str(notice)
+            ret += unicode(notice)
         return ret
+    def __str__(self):
+        return to_utf8(self.__unicode__())
 
     def xml(self, fileobj=None):
         msg = """<?xml version="1.0"?>\n<updates>"""
@@ -408,6 +419,9 @@ class UpdateMetadata(object):
 
 def main():
     """ update_md test function. """
+    import yum.misc
+
+    yum.misc.setup_locale()
     def usage():
         print >> sys.stderr, "Usage: %s <update metadata> ..." % sys.argv[0]
         sys.exit(1)
@@ -420,7 +434,7 @@ def main():
         um = UpdateMetadata()
         for srcfile in sys.argv[1:]:
             um.add(srcfile)
-        print um
+        print unicode(um)
     except IOError:
         print >> sys.stderr, "%s: No such file:\'%s\'" % (sys.argv[0],
                                                           sys.argv[1:])
